/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file           : mf_config.h
  * @brief          : Header for mf_config.c file.
  *                   This file contains the common defines of the application.
  ******************************************************************************
  * @attention
  *
  * Copyright (c) [2019] [Fudan Microelectronics]
  * THIS SOFTWARE is licensed under the Mulan PSL v1.
  * can use this software according to the terms and conditions of the Mulan PSL v1.
  * You may obtain a copy of Mulan PSL v1 at:
  * http://license.coscl.org.cn/MulanPSL
  * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND, EITHER EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT, MERCHANTABILITY OR FIT FOR A PARTICULAR
  * PURPOSE.
  * See the Mulan PSL v1 for more details.
  *
  ******************************************************************************
  */
/* USER CODE END Header */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __MF_CONFIG_H
#define __MF_CONFIG_H

#include "fm33le0xxa_fl.h"

#ifdef __cplusplus
extern "C" {
#endif

/* BIT MAP */
#define BIT0            0x00000001UL
#define BIT1            0x00000002UL
#define BIT2            0x00000004UL
#define BIT3            0x00000008UL
#define BIT4            0x00000010UL
#define BIT5            0x00000020UL
#define BIT6            0x00000040UL
#define BIT7            0x00000080UL
#define BIT8            0x00000100UL
#define BIT9            0x00000200UL
#define BIT10           0x00000400UL
#define BIT11           0x00000800UL
#define BIT12           0x00001000UL
#define BIT13           0x00002000UL
#define BIT14           0x00004000UL
#define BIT15           0x00008000UL
#define BIT16           0x00010000UL
#define BIT17           0x00020000UL
#define BIT18           0x00040000UL
#define BIT19           0x00080000UL
#define BIT20           0x00100000UL
#define BIT21           0x00200000UL
#define BIT22           0x00400000UL
#define BIT23           0x00800000UL
#define BIT24           0x01000000UL
#define BIT25           0x02000000UL
#define BIT26           0x04000000UL
#define BIT27           0x08000000UL
#define BIT28           0x10000000UL
#define BIT29           0x20000000UL
#define BIT30           0x40000000UL
#define BIT31           0x80000000UL

typedef union
{
  unsigned char B08;
  struct
  {
    unsigned char bit0:1;
    unsigned char bit1:1;
    unsigned char bit2:1;
    unsigned char bit3:1;
    unsigned char bit4:1;
    unsigned char bit5:1;
    unsigned char bit6:1;
    unsigned char bit7:1;
  }Bit;
}B08_Bit;

union B16_B08_2                 //С˴洢(51Ϊ)
{
    unsigned short  B16;
    unsigned char   B08[2];
};

union S16_B08_2                 //С˴洢(51Ϊ)
{
    unsigned char   B08[2];
    signed short    S16;
};

union B32_B08_2                 //С˴洢(51Ϊ)
{
    unsigned long   int B32;
    unsigned char   B08[4];
};

union B16_B08
{
    unsigned char   B08[2];
  //unsigned int B16;
};

union S16_B08
{
    unsigned char   B08[2];
  //unsigned int S16;
};

union B32_B08
{
    unsigned char   B08[4];
  //unsigned long int B32;
};




/* Flash_CRĴ޶Ӧ */
#define FLASH_Lock()   
#define FLASH_Unlock()  

/** 
  * @brief  FLASH Status  
  */

typedef enum
{ 
  FLASH_BUSY = 1,
  FLASH_ERROR_PG,
  FLASH_ERROR_WRP,
  FLASH_COMPLETE,
  FLASH_TIMEOUT
}FLASH_Status;


/*  */
#define __SYSTEM_CLOCK              (64000000)
#define __SYSTEM_CLOCK_1MHz_PRE     (__SYSTEM_CLOCK/1000000)
#define __SYSTEM_CLOCK_1KHz_PRE     (__SYSTEM_CLOCK/1000)
#define __SYSTEM_CLOCK_10KHz_PRE    (__SYSTEM_CLOCK/10000)

/* ʱز */
#define PLLDB_VALUE                 (__SYSTEM_CLOCK_1MHz_PRE - 1)
#define BSTIM32_AUTORELOAD_VALUE    5000    /* usΪλ,ʱԴAPB=AHB */
#define BSTIM16_AUTORELOAD_VALUE    1000    /* usΪλ,ʱԴAPB=AHB */
#define GPTIM0_AUTORELOAD_VALUE     0x10000 /* usΪλ,ʱԴAPB=AHB */
#define SPI2_BAUD                   FL_SPI_BAUDRATE_DIV16 /* ̶APB 16Ƶ */
#define I2C_BAUDRATE                40000  /* 40K */
#define I2C_BRG                     ((__SYSTEM_CLOCK/(2 * I2C_BAUDRATE)) - 1) /* I2CӦBRG */

#define VERIFY_SYSREG_TIMER         100000
#define VERIFY_CTRLDATA_PARA_TIMER  200000
#define MAIN_LOOP_TIMER_PERIOD      500000

/*  */
#define SLEEP_SUPPORT               1   /* Ƿ֧sleep 0-֧ 1-֧ */
#define EEPROM_SUPPORT              0   /* Ƿ֧EEprom */

/* жȼ */
#define ECCC_RAMP_FEDT_IRQ_PRI      0   /* ECC 洢ʱ ͣ ж */
#define SVD_IRQ_PRI                 2   /* SVD ж */
#define LIN_SLAVE_UART_IRQ_PRI      1   /* Lin Slave UARTж */
#define LIN_MASTER_UART_IRQ_PRI     1   /* Lin Master UARTж */
#define TAU_PGL_IRQ_PRI             2   /* Lin ʱʱж TAU13TAU07PWMж */
#define GPIO_IRQ_PRI                2   /* ʱж */
#define ADC_IRQ_PRI                 3   /* ADC ж */
#define DMA_IRQ_PRI                 3   /* DMA ж */
#define WKUPx_IRQ_PRI               2   /* WKUPx ж */
#define GPTIM0_IRQ_PRI              2   /* GPTIM0 ж */

#define ADC_Captuer_TIME   50    /* 200ms */

typedef enum
{
    LED_TEST = 0x1,
    I2C_TEST = 0x2,
    SPI_TEST = 0x4,
    CODE_FLASH_TEST = 0x8
    
}TEST_TYPE;


#define MESSAGE_FROM_LIN    1


/* Lin ͨ */
typedef union
{
    uint8_t data[8];
    struct
    {
        /* PWMƵԼռձ */
        uint32_t pwm_freq :24;  /* HzΪλΧ40Hz~10KHzܱ10KHzƵ(ƫ)0xFFFFFFάֲ */
        uint32_t pwm_duty :8;   /* Hex, 0~100 0xFFάֲ */
        
        uint32_t test_ctrl :8;  /* ο LED_TEST  */
        uint32_t rsvd :24;      /*  */
    }str;
}ctrl_data_type;

typedef struct
{
    ctrl_data_type ctrl_data;
    union B16_B08 crc16;
    uint8_t update_flag; /* 0-޸ 1-и */
    uint8_t message_src; /* ϢԴ 1-Lin */
}app_struct_type;

extern volatile app_struct_type g_app_struct;

/*============================EEPROM洢==========================================*/
#define         ee_ctrldata_page      0x00        /* ҳַ */
#define         ee_ctrldata_lenth       8

/*============================USERFLASH洢==========================================*/
#define USER_FLASH_BKUP_PAGE        2       /* pageԭʼλҳƫ */
#define USER_FLASH_CRC_PAGE         4       /* 2ֽCRC page */

#define     userflash_ctrldata_page         0x02        /* ҳַƫ */
#define     userflash_ctrldata_offset       0x00        /* ҳڵַƫ */
#define     userflash_ctrldata_lenth        8

/* Exported functions prototypes ---------------------------------------------*/
void GPIO_Get_InitParam(GPIO_Type* GPIOx, uint32_t pin, FL_GPIO_InitTypeDef* GPIO_InitStruct);
void AnalogIO( GPIO_Type* GPIOx, uint32_t PinNum );
void OutputIO( GPIO_Type* GPIOx, uint32_t PinNum, uint8_t outputType );
void InputIO( GPIO_Type* GPIOx, uint32_t PinNum, uint8_t pull );
void AltFunIO( GPIO_Type* GPIOx, uint32_t PinNum, uint8_t outputType, uint8_t pull,  uint8_t remapPin );
void CloseIO( GPIO_Type* GPIOx, uint32_t PinNum );
void Close_And_ClearIRQ(IRQn_Type IRQn);

void app_process(void);
void get_current_ctrl_status(ctrl_data_type *ctrl_data, uint32_t u32Delay_us);

void MF_Clock_Init(void);
void MF_SystemClock_Config(void);
void MF_Config_Init(void);
void Error_Handler(void);
void MF_PMU_Init(void);

typedef struct
{
  uint8_t length;  /* data length */  
  uint8_t Data[8]; /*!< Contains the data to be received. It ranges from 0 to 
                        0xFF. */
}BootRxMsg;

#ifdef __cplusplus
}
#endif

#endif /* __MF_CONFIG_H */

/************************ (C) COPYRIGHT FMSH *****END OF FILE****/
