/**    
  ******************************************************************************
  * @attention    
  * Copyright 2023 SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICRO.)
  *        
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met: 
  *    
  * 1. Redistributions of source code must retain the above copyright notice, 
  *    this list of conditions and the following disclaimer.
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software without
  *    specific prior written permission.
  *    
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS AS IS
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   
  * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
  * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT   
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
  *******************************************************************************
  */
  
/* Includes ------------------------------------------------------------------*/
#include "mf_config.h"
#include "fm33le0xxa_fl.h"
#include "clock.h"
#include "key.h"
#include "lin.h"
#include "adc.h"
#include "gptim.h"
#include "i2c.h"
#include "spi.h"
#include "flash.h"
#include "led.h"

/* Private function prototypes -----------------------------------------------*/

/* SPIӦ */
#define LENGTH      8
uint8_t rBuffer[LENGTH] = {0};
uint8_t wBuffer[LENGTH] = {0x12, 0x23, 0x45, 0x67, 0x89, 0x10, 0xAB, 0xCD};

/**
  * @brief  ȡ
  * @param  GPIOxGPIOA ~ GPIOD
            pinź
            GPIO_InitStructGPIOòṹ
  * @retval void
  */
void GPIO_Get_InitParam(GPIO_Type* GPIOx, uint32_t pin, FL_GPIO_InitTypeDef* GPIO_InitStruct)
{
    uint32_t pinbit = 0x00;
    uint32_t i,temp;
    
    for (i = 0x00; i < 0x10; i++)
    {
        pinbit = pin & (1 << i);
        if(pinbit > 0) break;                                         
    }
    
    if(i < 0x10)
    {
        pinbit = i;
        GPIO_InitStruct->pin = pin;
        
        /*  */
        if (GPIOx->PUEN & pin)
        {
            GPIO_InitStruct->pull = FL_ENABLE;
        }
        else
        {
            GPIO_InitStruct->pull = FL_DISABLE;
        }
        
        /* 쿪© */
        temp = (GPIOx->ODEN & pin)>>pinbit;                                   
        if(temp == (uint32_t)FL_GPIO_OUTPUT_OPENDRAIN)
        {                                         
            GPIO_InitStruct->outputType = FL_GPIO_OUTPUT_OPENDRAIN;
        }
        else
        {
            GPIO_InitStruct->outputType = FL_GPIO_OUTPUT_PUSHPULL;
        }  
        
        /* Ź */
        temp = (GPIOx->FCR >> (pinbit * 2)) & 0x00000003;                                    
        if(temp == (uint32_t)FL_GPIO_MODE_INPUT)
        {                                         
            GPIO_InitStruct->mode = FL_GPIO_MODE_INPUT;
        }
        else if(temp == (uint32_t)FL_GPIO_MODE_OUTPUT)
        {
            GPIO_InitStruct->mode = FL_GPIO_MODE_OUTPUT;
        }
        else if(temp == (uint32_t)FL_GPIO_MODE_DIGITAL)
        {                                         
            GPIO_InitStruct->mode = FL_GPIO_MODE_DIGITAL;
        }
        else
        {
            GPIO_InitStruct->mode = FL_GPIO_MODE_ANALOG;
        }

        /* ֹ */
        temp = (GPIOx->DFS & pin)>> pinbit;                                    
        if(temp == (uint32_t)FL_ENABLE)
        {                                         
            GPIO_InitStruct->remapPin = FL_ENABLE;
        }
        else if(temp == (uint32_t)FL_DISABLE)
        {
            GPIO_InitStruct->remapPin = FL_DISABLE;
        }
    }  
}


/**
  * @brief  ģ⹦:ADC
  * @param  GPIOxGPIOA ~ GPIOD
            PinNumź
  * @retval void
  */
void AnalogIO( GPIO_Type* GPIOx, uint32_t PinNum )
{
    FL_GPIO_InitTypeDef  GPIO_InitStructure;
    FL_GPIO_InitTypeDef  GPIO_InitStructureRun;
    
    GPIO_Get_InitParam(GPIOx, PinNum, &GPIO_InitStructureRun);
    
    if( (GPIO_InitStructureRun.pin                != PinNum) ||
        (GPIO_InitStructureRun.outputType        != FL_GPIO_OUTPUT_OPENDRAIN) ||
        (GPIO_InitStructureRun.pull                != FL_DISABLE) ||
        (GPIO_InitStructureRun.mode                != FL_GPIO_MODE_ANALOG) ||
        (GPIO_InitStructureRun.remapPin            != FL_DISABLE ))
    {
        /* ̶©ֹ */
        GPIO_InitStructure.pin = PinNum;
        GPIO_InitStructure.outputType = FL_GPIO_OUTPUT_OPENDRAIN;
        GPIO_InitStructure.pull = FL_DISABLE;
        GPIO_InitStructure.mode = FL_GPIO_MODE_ANALOG;
        GPIO_InitStructure.remapPin =FL_DISABLE;       
        
        FL_GPIO_Init(GPIOx, &GPIO_InitStructure);        
    }
}


/**
  * @brief  IO
  * @param  GPIOxGPIOA ~ GPIOD
            PinNumź
            pullѡ
  * @retval void
  */
void InputIO( GPIO_Type* GPIOx, uint32_t PinNum, uint8_t pull )
{
    FL_GPIO_InitTypeDef  GPIO_InitStructure;
    FL_GPIO_InitTypeDef  GPIO_InitStructureRun;
    
    GPIO_Get_InitParam(GPIOx, PinNum, &GPIO_InitStructureRun);
    
    if( (GPIO_InitStructureRun.pin                != PinNum) ||
        (GPIO_InitStructureRun.outputType        != FL_GPIO_OUTPUT_OPENDRAIN) ||
        (GPIO_InitStructureRun.pull                != pull) ||
        (GPIO_InitStructureRun.mode                != FL_GPIO_MODE_INPUT) ||
        (GPIO_InitStructureRun.remapPin            != FL_DISABLE) )    
    {
        /* ̶©ģʽ */
        GPIO_InitStructure.pin = PinNum;
        GPIO_InitStructure.outputType = FL_GPIO_OUTPUT_OPENDRAIN;
        GPIO_InitStructure.pull = pull;
        GPIO_InitStructure.mode = FL_GPIO_MODE_INPUT;
        GPIO_InitStructure.remapPin = FL_DISABLE;        
        
        FL_GPIO_Init(GPIOx, &GPIO_InitStructure);    
    }    
}


/**
  * @brief  IO
  * @param  GPIOxGPIOA ~ GPIOD
            PinNumź
            outputType
  * @retval void
  */
void OutputIO( GPIO_Type* GPIOx, uint32_t PinNum, uint8_t outputType )
{
    FL_GPIO_InitTypeDef  GPIO_InitStructure;
    FL_GPIO_InitTypeDef  GPIO_InitStructureRun;
    
    GPIO_Get_InitParam(GPIOx, PinNum, &GPIO_InitStructureRun);

    if( (GPIO_InitStructureRun.pin                != PinNum) ||
        (GPIO_InitStructureRun.outputType        != outputType) ||
        (GPIO_InitStructureRun.pull                != FL_DISABLE) ||
        (GPIO_InitStructureRun.mode                != FL_GPIO_MODE_OUTPUT) ||
        (GPIO_InitStructureRun.remapPin            != FL_DISABLE) )    
    {
        /* ̶ģʽֹ */
        GPIO_InitStructure.pin = PinNum;
        GPIO_InitStructure.outputType = outputType;
        GPIO_InitStructure.pull = FL_DISABLE;
        GPIO_InitStructure.mode = FL_GPIO_MODE_OUTPUT;
        GPIO_InitStructure.remapPin = FL_DISABLE;
         
        FL_GPIO_Init(GPIOx, &GPIO_InitStructure);        
    }
}


/**
  * @brief  ⹦ܿ
  * @param  GPIOxGPIOA ~ GPIOD
            PinNumź
            outputType
            pullѡ
            remapPinӳѡ
  * @retval void
  */
void AltFunIO( GPIO_Type* GPIOx, uint32_t PinNum, uint8_t outputType, uint8_t pull,  uint8_t remapPin )
{                                                                
    FL_GPIO_InitTypeDef  GPIO_InitStructure;
    FL_GPIO_InitTypeDef  GPIO_InitStructureRun;
    
    GPIO_Get_InitParam(GPIOx, PinNum, &GPIO_InitStructureRun);
    
    if( (GPIO_InitStructureRun.pin                != PinNum) ||
        (GPIO_InitStructureRun.outputType        != outputType) ||
        (GPIO_InitStructureRun.pull                != pull) ||
        (GPIO_InitStructureRun.mode                != FL_GPIO_MODE_DIGITAL) ||
        (GPIO_InitStructureRun.remapPin            != remapPin) )    
    {
        /* ̶ģʽ */
        GPIO_InitStructure.pin = PinNum;
        GPIO_InitStructure.outputType = outputType;
        GPIO_InitStructure.pull = pull;
        GPIO_InitStructure.mode = FL_GPIO_MODE_DIGITAL;
        GPIO_InitStructure.remapPin = remapPin;
         
        FL_GPIO_Init(GPIOx, &GPIO_InitStructure);        
    }
}


/**
  * @brief  رIO빦ֹ
  * @param  GPIOxGPIOA ~ GPIOD
            PinNumź
  * @retval void
  */
void CloseIO( GPIO_Type* GPIOx, uint32_t PinNum )
{
    FL_GPIO_DeInit(GPIOx, PinNum);/* עSWDӿڵPD7,8ıǵý޷ */
}


/**
  * @brief  NVICжϹر
  * @param  void
  * @retval void
  */
void Close_And_ClearIRQ(IRQn_Type IRQn)
{
    NVIC_DisableIRQ(IRQn);
    NVIC_ClearPendingIRQ(IRQn);	
}

/**
  * @brief  The application entry point.
  * @retval int
  */
void MF_Clock_Init(void)
{
  /* MCU Configuration--------------------------------------------------------*/
   FL_RCC_EnableGroup1BusClock(FL_RCC_GROUP1_BUSCLK_RTC);
   FL_RTC_WriteAdjustValue(RTC, 0);
   FL_RCC_DisableGroup1BusClock(FL_RCC_GROUP1_BUSCLK_RTC);
  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */

  /* System interrupt init*/

  /* Initialize all configured peripherals */
}



/**
  * @brief System Clock Configuration
  * @retval None
  */
void MF_SystemClock_Config(void)
{
    
}


/**
  * @brief  PMU ʼ
  * @param  void
  * @retval void
  */
void MF_PMU_Init(void)
{
    FL_PMU_SleepInitTypeDef    defaultInitStruct= {FL_DISABLE};
    
    /* ʹõSVDΪԴʱPMU_CRֻNormal Sleepģʽ */
    defaultInitStruct.deepSleep       = FL_PMU_SLEEP_MODE_NORMAL;  /* Normal Sleep */ 
    defaultInitStruct.wakeupFrequency = FL_PMU_RCHF_WAKEUP_FREQ_8MHZ;
    defaultInitStruct.wakeupDelay     = FL_PMU_WAKEUP_DELAY_2US;

    (void)FL_PMU_Sleep_Init(PMU, &defaultInitStruct);  
}


/**
  * @brief  ûӦʼ
  * @param  void
  * @retval void
  */
void MF_Config_Init(void)
{
    /* ûѺĹ  */
    MF_PMU_Init();

    /* ȽⲿXTHFƵ1MPLL໷Ƶ64M */
    (void)SelXTHFToPLL(&XTHF_InitParameter, &PLL_InitParameter);
    
    /* IWDT ߶ */
    FL_IWDT_EnableFreezeWhileSleep(IWDT);
    
    /* ָʾƳʼ */
    KEY_Init();
    
    /* LIN ʼ */
    lin_init();
    
    /* ADC ʼ */
    AdcInit();
    
    /* ADC ԴBSTIM32ʼ */
    BSTIM32_Init();
    
    /* GPTIM0 벶ʼ */
    GPTIM0_IC_Init();
    
    /* GPTIM1 PWMʼ */
    GPTIM1_PWM_Init();
    
    /* I2C ʼ */
    I2C_Init();
    
    /* SPI2 ʼ */
    MF_SPI_Init(SPI2);
    
    /* LED ʼ */
    LED_Iint();
    
}


/**
  * @brief  ָȡ PWM Ƶʼռձ
  * @param  ctrl_dataָṹ
            u32Delay_usʱ
  * @retval void
  */
void get_current_ctrl_status(ctrl_data_type *ctrl_data, uint32_t u32Delay_us)
{
    uint32_t frequency;
    uint8_t duty;
    
    /* ȡǰPWM״̬ */
    if (0 == u32Delay_us)
    {
        u32Delay_us += 1;
    }
    
    while (get_pwm_status(&frequency, &duty) && u32Delay_us)
    {
        if (--u32Delay_us)
        {
            FL_DelayUs(1);
        }
    }
    
    if (0 == u32Delay_us)
    {
        ctrl_data->str.pwm_freq = 0;
        ctrl_data->str.pwm_duty = 0;        
    }
    else
    {
        ctrl_data->str.pwm_freq = frequency;
        ctrl_data->str.pwm_duty = duty;         
    }
    
}


/**
  * @brief  ûӦô
  * @param  void
  * @retval void
  */
void app_process(void)
{
    app_struct_type temp_app_struct;
    uint32_t primask;    
    
    temp_app_struct.update_flag = 0;
    
    /* жȫֱ̬ */
    primask = __get_PRIMASK();
    __disable_irq();
    if (g_app_struct.update_flag)
    {
        memcpy((void *)&temp_app_struct, (void *)&g_app_struct, sizeof(app_struct_type));
        
        g_app_struct.update_flag = 0;
    }
    __set_PRIMASK(primask);
    
    /* жǷ */
    if (temp_app_struct.update_flag)
    {
        /* 洢µĲ EEprom */
        #if (EEPROM_SUPPORT == 1)
        WriteE2WithBackup(I2C, ee_ctrldata_page, (uint8_t *)g_app_struct.ctrl_data.data, ee_ctrldata_lenth );
        #endif
        WriteUserFlashWithBackup( userflash_ctrldata_page, userflash_ctrldata_offset, (uint8_t *)g_app_struct.ctrl_data.data, userflash_ctrldata_lenth );
        
        
        /* PWM */
        if ((0xFFFFFF != temp_app_struct.ctrl_data.str.pwm_duty) || 
            (0xFF != temp_app_struct.ctrl_data.str.pwm_freq))
        {
            SET_PWM_Para(temp_app_struct.ctrl_data.str.pwm_freq, temp_app_struct.ctrl_data.str.pwm_duty);
            clear_pwm_update_flag();
        }
        
        /*  TEST  */
        /* LED  */
        if (temp_app_struct.ctrl_data.str.test_ctrl & LED_TEST)
        {
            LED_TEST_TEST();
        }
        /* I2Cд */
        if (temp_app_struct.ctrl_data.str.test_ctrl & I2C_TEST)
        {
//            (void)I2C_Eeprom();
            LED_IIC_TEST();
        }
        /* SPI2 д */
        if (temp_app_struct.ctrl_data.str.test_ctrl & SPI_TEST)
        {
            SpiWriteData(SPI2, wBuffer, LENGTH);
            SpiReadData(SPI2, rBuffer, LENGTH);
            LED_SPI_TEST();
        }
        /* Flash дʾ */
        if (temp_app_struct.ctrl_data.str.test_ctrl & CODE_FLASH_TEST)
        {
            USER_Flash_RW();
            LED_USER_FLASH_TEST();
        }
        
        get_current_ctrl_status(&temp_app_struct.ctrl_data, 1000);/* ȴ1000us */
        
        if (MESSAGE_FROM_LIN == temp_app_struct.message_src)
        {
            memcpy(LIN_SLAVE_CTRL_RESPONSE_Data, temp_app_struct.ctrl_data.data, 8);
        }
    }
    
    /* ADC Ӧʾ */
    GetADC_DMA_Sample();
    
    /* Ӧʾ */
    KEY_Process();
    
    /* ϵͳָʾ */
    LED_RUN_TEST();
}



/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
    /* User can add his own implementation to report the HAL error return state */

  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{ 
  /* USER CODE BEGIN Assert_Failed */
    /* User can add his own implementation to report the file name and line number,
       tex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END Assert_Failed */
}
#endif /* USE_FULL_ASSERT */

/************************ (C) COPYRIGHT FMSH *****END OF FILE****/
