/**    
  ******************************************************************************
  * @attention    
  * Copyright 2023 SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICRO.)
  *        
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met: 
  *    
  * 1. Redistributions of source code must retain the above copyright notice, 
  *    this list of conditions and the following disclaimer.
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software without
  *    specific prior written permission.
  *    
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS AS IS
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   
  * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
  * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT   
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
  *******************************************************************************
  */
#include "adc.h"
#include "SEGGER_RTT.h"


#define ADC_EXTERNAL_GROUP  FL_ADC_EXTERNAL_CH3|FL_ADC_EXTERNAL_CH4|FL_ADC_EXTERNAL_CH5|\
                            FL_ADC_EXTERNAL_CH7|FL_ADC_EXTERNAL_CH11|FL_ADC_INTERNAL_AVREF
#define ADC_DMA_LENGTH 12


volatile uint16_t  ADC_DMA_Result[2][ADC_DMA_LENGTH/2] ={0};    /* ADCԭʼֵ */
uint16_t  use_ADC_DMA_Result[2][ADC_DMA_LENGTH/2] ={0};         /* ADCֵû */
uint32_t GetValue[ADC_DMA_LENGTH/2] = {0};                      /* ADCʵʵѹֵ */

uint8_t ADCSequence_OVERFLAG=0;                                 /* ADCв־ */


/**
  * @brief  ADCжϷ
  * @param  void
  * @retval void
  */
void ADC_IRQHandler(void)
{
    if(FL_ADC_IsActiveFlag_EndOfSequence(ADC))
    {
        FL_ADC_ClearFlag_EndOfSequence(ADC);
    }
    
}


/**
  * @brief  DMAжϷ
  * @param  void
  * @retval void
  */
void DMA_IRQHandler(void)
{
    if(FL_DMA_IsActiveFlag_TransferComplete(DMA,FL_DMA_CHANNEL_4))      /* ADCʧܣֹADCȡýǰݽ룬ԭʼ ʲôƿԱ֤ADCȡõʵʱ*/
    {
        FL_DMA_ClearFlag_TransferComplete(DMA,FL_DMA_CHANNEL_4);        /* ȫжϱ־ */
        memcpy(use_ADC_DMA_Result,(const void*)ADC_DMA_Result,sizeof (ADC_DMA_Result));
        ADCSequence_OVERFLAG|=1<<1;
    }

    if(FL_DMA_IsActiveFlag_TransferHalfComplete(DMA,FL_DMA_CHANNEL_4))
    {
        FL_DMA_ClearFlag_TransferHalfComplete(DMA,FL_DMA_CHANNEL_4);    /* жϱ־ */
        ADCSequence_OVERFLAG|=1;
    }

    if(FL_DMA_IsActiveFlag_AddressError(DMA))
    {
        FL_DMA_ClearFlag_AddressError(DMA);                             /* ַ־ */
    }
}


/**
  * @brief  ADCDMAֵͨȡ
  * @param  void
  * @retval void
  */
void  GetADC_DMA_Sample(void)
{
    uint8_t i;
    
    if(ADCSequence_OVERFLAG==0x03)
    {
        /* в־ */
        ADCSequence_OVERFLAG=0;
        
        /* ͨȡתֵ */
        for(i=0;i<(sizeof(GetValue)/4-1);i++)
        {
            /* ȡѹֵλmv */
            GetValue[i]  =  (uint32_t)(((uint64_t)use_ADC_DMA_Result[0][i] * (ADC_VREF)) / ((uint64_t)use_ADC_DMA_Result[0][sizeof(GetValue)/4-1] * 10));
            
        }
        
    }
}


/**
  * @brief  ADCDMA
  * @param  buffer: ָ 
  *         length: 䳤
  * @retval void
  */
void ADC_DMA(uint16_t *buffer , uint32_t length)
{
    FL_DMA_InitTypeDef       DMA_InitStruct = {0};
    FL_DMA_ConfigTypeDef     DMA_ConfigStruct;
    
    DMA_InitStruct.periphAddress        = FL_DMA_PERIPHERAL_FUNCTION1;
    DMA_InitStruct.direction            = FL_DMA_DIR_PERIPHERAL_TO_RAM;
    DMA_InitStruct.memoryAddressIncMode = FL_DMA_MEMORY_INC_MODE_INCREASE;
    DMA_InitStruct.dataSize             = FL_DMA_BANDWIDTH_16B;
    DMA_InitStruct.priority             = FL_DMA_PRIORITY_HIGH;
    DMA_InitStruct.circMode             = FL_ENABLE;
    FL_DMA_Init(DMA, &DMA_InitStruct, FL_DMA_CHANNEL_4);
    
    FL_DMA_Enable(DMA);
    
    DMA_ConfigStruct.memoryAddress      = (uint32_t)buffer;
    DMA_ConfigStruct.transmissionCount  = length - 1;
    FL_DMA_StartTransmission(DMA, &DMA_ConfigStruct, FL_DMA_CHANNEL_4);
    
    /* DMAж */
    /* ־ */
    FL_DMA_ClearFlag_TransferComplete(DMA,FL_DMA_CHANNEL_4);     /* ȫжϱ־ */
    FL_DMA_ClearFlag_TransferHalfComplete(DMA,FL_DMA_CHANNEL_4); /* жϱ־ */

    FL_DMA_EnableIT_TransferComplete(DMA,FL_DMA_CHANNEL_4);      /* ʹȫж*/
    FL_DMA_EnableIT_TransferHalfComplete(DMA,FL_DMA_CHANNEL_4);  /* ʹܰж*/

    /* ʹܲNVICNVIC */
    NVIC_DisableIRQ(DMA_IRQn);
    NVIC_SetPriority(DMA_IRQn, DMA_IRQ_PRI); /* жȼ */
    NVIC_EnableIRQ(DMA_IRQn);
}



/**
  * @brief  ADCʼ
  * @param  void
  * @retval void
  */
void AdcInit(void)
{
    FL_GPIO_InitTypeDef         GPIO_InitStruct = {0};
    FL_ADC_CommonInitTypeDef    ADC_CommonInitStruct;
    FL_ADC_InitTypeDef          ADC_InitStruct ;
    
    /* Ϊģ⹦ PC6 IN11*/
    GPIO_InitStruct.pin        = FL_GPIO_PIN_6;
    GPIO_InitStruct.mode       = FL_GPIO_MODE_ANALOG;
    GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
    GPIO_InitStruct.pull       = FL_DISABLE;
    GPIO_InitStruct.remapPin   = FL_DISABLE;
    FL_GPIO_Init( GPIOC, &GPIO_InitStruct );
    
    /* Ϊģ⹦ PA14 IN7*/
    GPIO_InitStruct.pin        = FL_GPIO_PIN_14;
    GPIO_InitStruct.mode       = FL_GPIO_MODE_ANALOG;
    GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
    GPIO_InitStruct.pull       = FL_DISABLE;
    GPIO_InitStruct.remapPin   = FL_DISABLE;
    FL_GPIO_Init( GPIOA, &GPIO_InitStruct );
    
    /* Ϊģ⹦ PD0 IN3 \PD1 IN4 \PD2 IN5 */
    GPIO_InitStruct.pin        = FL_GPIO_PIN_0|FL_GPIO_PIN_1|FL_GPIO_PIN_2;
    GPIO_InitStruct.mode       = FL_GPIO_MODE_ANALOG;
    GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
    GPIO_InitStruct.pull       = FL_DISABLE;
    GPIO_InitStruct.remapPin   = FL_DISABLE;
    FL_GPIO_Init( GPIOD, &GPIO_InitStruct );
    
    /* ADC ʱ */
    ADC_CommonInitStruct.clockSource         = FL_RCC_ADC_CLK_SOURCE_PLL;        /* ADC_CLK ʱԴ */
    ADC_CommonInitStruct.clockPrescaler      = FL_RCC_ADC_PSC_DIV4;              /* ADC_CLK Ƶϵ */
    FL_ADC_CommonInit(&ADC_CommonInitStruct);
    
    /* ADC Ĵ */
    ADC_InitStruct.conversionMode           = FL_ADC_CONV_MODE_SINGLE;     /* ģʽ */
    ADC_InitStruct.autoMode                 = FL_ADC_SINGLE_CONV_MODE_AUTO;/* ȫԶ */
    ADC_InitStruct.waitMode                 = FL_DISABLE;                    /* ȴ */
    ADC_InitStruct.overrunMode              = FL_ENABLE;                   /* ϴ */
    ADC_InitStruct.scanDirection            = FL_ADC_SEQ_SCAN_DIR_FORWARD; /* ͨɨ */
    ADC_InitStruct.externalTrigConv         = FL_ADC_TRIGGER_EDGE_RISING;    /* źΪ */
    ADC_InitStruct.triggerSource            = FL_ADC_TRGI_BSTIM;             /* źΪBSTIM */
    ADC_InitStruct.fastChannelTime          = FL_ADC_FAST_CH_SAMPLING_TIME_2_ADCCLK;  /* ͨʱ */
    ADC_InitStruct.lowChannelTime           = FL_ADC_SLOW_CH_SAMPLING_TIME_192_ADCCLK;/* ͨʱ */
    ADC_InitStruct.oversamplingMode         = FL_ENABLE;                   /*  رտ̲ʱ */
    ADC_InitStruct.overSampingMultiplier    = FL_ADC_OVERSAMPLING_MUL_8X;  /* 8 */
    ADC_InitStruct.oversamplingShift        = FL_ADC_OVERSAMPLING_SHIFT_3B;/* */
   
    FL_ADC_Init(ADC, &ADC_InitStruct);
    
    FL_ADC_EnableDMAReq(ADC);
    
    FL_VREF_EnableVREFBuffer(VREF);                              /* ʹVREF BUFFER */
    
    FL_ADC_DisableSequencerChannel(ADC, ADC_EXTERNAL_GROUP);
    FL_ADC_EnableSequencerChannel(ADC, ADC_EXTERNAL_GROUP);      /* ⲿͨѡ */
    
    ADC_DMA((uint16_t *)(ADC_DMA_Result), ADC_DMA_LENGTH);
    FL_ADC_Enable(ADC);
    
    /* ADCж */
    /* ־ */
    FL_ADC_ClearFlag_EndOfConversion(ADC);                       /* ־ */
    
    /* жʹ */
    FL_ADC_EnableIT_EndOfSequence(ADC);                  /* ʹж*/
    FL_ADC_EnableIT_Overrun(ADC);                      /* ʹݳͻж*/
    FL_ADC_EnableIT_EndOfConversion(ADC);                /* ʹתж*/

    /* ʹܲNVICNVIC */
    NVIC_DisableIRQ(ADC_IRQn);
    NVIC_SetPriority(ADC_IRQn, ADC_IRQ_PRI);                       /* жȼ */
    NVIC_EnableIRQ(ADC_IRQn);
}



/**
  * @brief  ADCתԴBSTIM_TRGO
  * @param  void 
  * @retval void
  */
void BSTIM32_Init(void)
{
    /* öʱΪ1s */
    FL_BSTIM32_InitTypeDef    defaultInitStruct;

    defaultInitStruct.prescaler       = 63999;
    defaultInitStruct.autoReload      = ADC_Captuer_TIME;
    defaultInitStruct.autoReloadState = FL_ENABLE;
    defaultInitStruct.clockSource     = FL_RCC_BSTIM32_CLK_SOURCE_APB2CLK;

    FL_BSTIM32_Init(BSTIM32,&defaultInitStruct );

    FL_BSTIM32_SetTriggerOutput(BSTIM32,FL_BSTIM32_TRGO_UPDATE);    
    FL_BSTIM32_ClearFlag_Update(BSTIM32);                               /* жϱ־λ */
    
    /*---------------- ж ----------------*/
    /* ־ */
    FL_BSTIM32_ClearFlag_Update(BSTIM32);
    /* жʹ */
    FL_BSTIM32_EnableIT_Update(BSTIM32);

    /* ʹܲNVIC */
    NVIC_DisableIRQ(BSTIM_IRQn);
    NVIC_SetPriority(BSTIM_IRQn, 2); /* жȼ */
    NVIC_EnableIRQ(BSTIM_IRQn);
    /*---------------------------------------------*/

    /* ʹLPTIM32 */
    FL_BSTIM32_Enable(BSTIM32);
    

}



/**
  * @brief  ADCģ鸴λ
  * @param  void
  * @retval void
  */
void ADC_DeInit(void)
{
    /* ָüĴλֵ */
    FL_ADC_CommonDeInit();
    
    /* ָĴλֵ */ 
    FL_ADC_DeInit(ADC);
    
    /* жϹ */
    NVIC_ClearPendingIRQ(ADC_IRQn);
    
    /* رж */
    NVIC_DisableIRQ(ADC_IRQn);
}


#if (SLEEP_SUPPORT == 1)

/**
  * @brief  ADCǰʼ
  * @param  void
  * @retval void
  */
void ADC_Sleep_Init(void)
{
    /* رģBUFFER */
    FL_VREF_DisableVREFBuffer(VREF);
    
    /* رշǻ */
    FL_BSTIM32_Disable(BSTIM32);  /* ܶʱ  TODO Ҫ֤رʱ */  
    FL_BSTIM32_DisableIT_Update(BSTIM32);
    FL_BSTIM32_ClearFlag_Update(BSTIM32);
    Close_And_ClearIRQ(BSTIM_IRQn);
    NVIC_ClearPendingIRQ(BSTIM_IRQn);/* 뱣BSTIM32NVICʹ */

    FL_ADC_Disable(ADC);
    FL_ADC_DisableIT_EndOfSequence(ADC);
    FL_ADC_ClearFlag_EndOfSequence(ADC);
    Close_And_ClearIRQ(ADC_IRQn);
    
    FL_DMA_DisableChannel(DMA, FL_DMA_CHANNEL_4);
    FL_DMA_DisableIT_TransferComplete(DMA,FL_DMA_CHANNEL_4);      /* ȫж*/
    FL_DMA_DisableIT_TransferHalfComplete(DMA,FL_DMA_CHANNEL_4);  /* ܰж*/ 
    FL_DMA_ClearFlag_TransferComplete(DMA,FL_DMA_CHANNEL_4);     /* ȫжϱ־ */
    FL_DMA_ClearFlag_TransferHalfComplete(DMA,FL_DMA_CHANNEL_4); /* жϱ־ */
    Close_And_ClearIRQ(DMA_IRQn);  
    
    /* رշǻIO */
    CloseIO(GPIOD, FL_GPIO_PIN_0);  /* PD0 --> ADC CH3 */
    CloseIO(GPIOD, FL_GPIO_PIN_1);  /* PD1 --> ADC CH4 */
    CloseIO(GPIOD, FL_GPIO_PIN_2);  /* PD2 --> ADC CH5 */
    CloseIO(GPIOA, FL_GPIO_PIN_14); /* PA14 --> ADC CH7 */
    CloseIO(GPIOC, FL_GPIO_PIN_6);  /* PC6 --> ADC CH11 */
    
    
    /* Դ */
}

/**
  * @brief  ADC Ѻʼ
  * @param  0-ͨ 1-ΪλѵԴ
  * @retval void
  */
void ADC_Awake_Init(uint8_t awake_type)
{
    FL_VREF_EnableVREFBuffer(VREF);                              /* ʹVREF BUFFER */
    /* 򿪷ǻIO */
    AnalogIO(GPIOD, FL_GPIO_PIN_0);  /* PD0 --> ADC CH3 */
    AnalogIO(GPIOD, FL_GPIO_PIN_1);  /* PD1 --> ADC CH4 */
    AnalogIO(GPIOD, FL_GPIO_PIN_2);  /* PD2 --> ADC CH5 */
    AnalogIO(GPIOA, FL_GPIO_PIN_14); /* PA14 --> ADC CH7 */
    AnalogIO(GPIOC, FL_GPIO_PIN_6);  /* PC6 --> ADC CH11 */
    
    /* ʼ */
    memset(ADC_DMA_Result, 0, sizeof(ADC_DMA_Result));

    /* Դ */
    
    /* ǻ账 */
    FL_ADC_ClearFlag_EndOfSequence(ADC);
    FL_ADC_EnableIT_EndOfSequence(ADC);
    NVIC_EnableIRQ(ADC_IRQn);
    FL_ADC_Enable(ADC);
    
    FL_DMA_ClearFlag_TransferComplete(DMA,FL_DMA_CHANNEL_4);     /* ȫжϱ־ */
    FL_DMA_ClearFlag_TransferHalfComplete(DMA,FL_DMA_CHANNEL_4); /* жϱ־ */
    FL_DMA_EnableIT_TransferComplete(DMA,FL_DMA_CHANNEL_4);      /* ȫж*/
    FL_DMA_EnableIT_TransferHalfComplete(DMA,FL_DMA_CHANNEL_4);  /* ܰж*/ 
    NVIC_EnableIRQ(DMA_IRQn);    
    FL_DMA_EnableChannel(DMA, FL_DMA_CHANNEL_4);
    
    FL_BSTIM32_ClearFlag_Update(BSTIM32);
    FL_BSTIM32_EnableIT_Update(BSTIM32);
    NVIC_EnableIRQ(BSTIM_IRQn); 
    FL_BSTIM32_Enable(BSTIM32);  /* ʹܶʱ  TODO Ҫ֤رʱ */ 
    
}

/**
  * @brief  ADC sleepж
  * @param  void
  * @retval 1: Sleep
  *         0: Wake
  */
uint8_t ADC_IS_Sleep(void)
{
    if (!FL_ADC_IsActiveFlag_Busy(ADC))
    {
        return 1;
    }
    else
    {
        return 0;
    }
}

#endif

