/**    
  ******************************************************************************
  * @attention    
  * Copyright 2023 SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICRO.)
  *        
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met: 
  *    
  * 1. Redistributions of source code must retain the above copyright notice, 
  *    this list of conditions and the following disclaimer.
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software without
  *    specific prior written permission.
  *    
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS AS IS
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   
  * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
  * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT   
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
  *******************************************************************************
  */
#include "clock.h"

/* 10ms ڱ־ */
volatile uint32_t Period10msFlag = 0;

#define PLL_TIMEOUT            0x00000FFFU  /* Լ90ms */ 
#define XTHF_TIMEOUT           0x0003FFFFU  /* Լ10s */

/* XTHFĴ */
#define XTHF_REG_CHECK(xthf)  ((FL_RCC_XTHF_ReadDriverStrength() == (xthf)->DriverStrength) &&\
                               (FL_RCC_XTHF_IsEnabled()          == FL_ENABLE))

/* ϵͳʱѡ */
volatile uint32_t SYSCLKSELErrorState = 0;
/* ϵͳʱ״̬ */
volatile SYSCLK_STATE SYSCLKState = SYSCLK_NORMAL;

/**
  * @brief  ϵͳʱIO:
  *         ΪXTLFRCLPRCHFAHB
  * @param  void
  * @retval void
  */
void FoutInit(void)
{
    FL_GPIO_InitTypeDef GPIO_InitStruct = {0};
    
    GPIO_InitStruct.pin             = FL_GPIO_PIN_12;
    GPIO_InitStruct.mode            = FL_GPIO_MODE_DIGITAL;
    GPIO_InitStruct.outputType      = FL_GPIO_OUTPUT_PUSHPULL;
    GPIO_InitStruct.pull            = FL_DISABLE;
    GPIO_InitStruct.remapPin        = FL_DISABLE;
    FL_GPIO_Init(GPIOB, &GPIO_InitStruct);
}


/* RCHFʱӲ */
const FL_RCHF_InitTypeDef RCHF_InitParameter = {
    
    /* RCHFʱƵ Ϊ8M,16M,24M*/
    FL_RCC_RCHF_FREQUENCY_8MHZ
    
};



/* XTHFʱӲ */
const FL_XTHF_InitTypeDef XTHF_InitParameter = {
    
    /* XTHFƵ 8M */
    8000000,
  
    /* XTHFǿ ƼֵΪ0x07 */
    0x07
};


/* PLLʱӲ */
const FL_PLL_InitTypeDef PLL_InitParameter = {

    /* PLLƵ Ƶ64M */
    64,
  
    /* PLLοʱԤƵ 8M ʱԴƵ 1M */
    FL_RCC_PLL_PSC_DIV8,
  
    /* PLLʱԴѡ */
    FL_RCC_PLL_CLK_SOURCE_XTHF
};

/**
  * @brief  BSTIMжϷ
  * @param  void
  * @retval void
  */
void BSTIM_IRQHandler(void)
{
    if(FL_BSTIM32_IsEnabledIT_Update(BSTIM32) && FL_BSTIM32_IsActiveFlag_Update(BSTIM32))
    {
        FL_BSTIM32_ClearFlag_Update(BSTIM32);

        if((0x1U == FL_FDET_IsActiveFlag_XTHFFail()) && (SYSCLK_ERROR != SYSCLKState))
        {
            /* лϵͳʱӣXTHFͣʶ溯ִ */
            SwtichToSenondarySystemClock(&RCHF_InitParameter, &PLL_InitParameter);
        }

        Period10msFlag = 1;
    }
}

/**
  * @brief  ͣжϷ
  * @param  void
  * @retval void
  */
void HFDET_IRQHandler(void)
{
    /* ͣжϴ */
    if((FL_FDET_IsEnabledIT_XTHFFail() == 0x01UL) && (FL_FDET_IsActiveFlag_XTHFFail() == 0x01UL))
    {
        
        /* XTHFͣʶPLLʧʶ溯ִ */
        SwtichToSenondarySystemClock(&RCHF_InitParameter, &PLL_InitParameter);
        
    }
}


/**
  * @brief
  * @rmtoll   PLLCR    OSEL    LL_RCC_GetPLLDigitalDomainOutput
  * @param    Source This parameter can be one of the following values:
  *           @arg @ref FL_RCC_PLL_CLK_SOURCE_RCHF
  *           @arg @ref FL_RCC_PLL_CLK_SOURCE_XTHF
  * @param    PLL_R ໷ĲοʱҪԤƵ1MٽPLLƵ This parameter can be one of the following values:
  *           @arg @ref FL_RCC_PLL_PSC_DIV1
  *           @arg @ref FL_RCC_PLL_PSC_DIV2
  *           @arg @ref FL_RCC_PLL_PSC_DIV4
  *           @arg @ref FL_RCC_PLL_PSC_DIV8
  *           @arg @ref FL_RCC_PLL_PSC_DIV12
  *           @arg @ref FL_RCC_PLL_PSC_DIV16
  *           @arg @ref FL_RCC_PLL_PSC_DIV24
  *           @arg @ref FL_RCC_PLL_PSC_DIV32
  * @param    PLL_DB This parameter can be Between parameters of the following values:
  *           @arg @ref

  * @param    PLL_O This parameter can be one of the following values:
  *           @arg @ref FL_RCC_PLL_OUTPUT_X1
  *           @arg @ref FL_RCC_PLL_OUTPUT_X2
  * @retval   None
  */
void RCC_PLL_ConfigDomain_SYS(uint32_t Source, uint32_t PLL_R, uint32_t PLL_DB, uint32_t PLL_O)
{
    MODIFY_REG(RCC->PLLCR, RCC_PLLCR_DB_Msk | RCC_PLLCR_REFPRSC_Msk | RCC_PLLCR_OSEL_Msk | RCC_PLLCR_INSEL_Msk,
               ((PLL_DB - 1 )<< RCC_PLLCR_DB_Pos) | PLL_R | PLL_O | Source);
}


/**
  * @brief  ڲRCHFʼ
  * @param  clock ĿƵ
  * @retval none
  */

static uint32_t RCHFInit(uint32_t RCHFclock)
{
    /* PLLƵ1MĲ */
    uint32_t PLL_Prescaler_1M;
    switch(RCHFclock)
    {
        case FL_RCC_RCHF_FREQUENCY_8MHZ:
            FL_RCC_RCHF_WriteTrimValue(RCHF8M_TRIM);
            PLL_Prescaler_1M = FL_RCC_PLL_PSC_DIV8;
            break;

        case FL_RCC_RCHF_FREQUENCY_16MHZ:
            FL_RCC_RCHF_WriteTrimValue(RCHF16M_TRIM);
            PLL_Prescaler_1M = FL_RCC_PLL_PSC_DIV16;
            break;

        case FL_RCC_RCHF_FREQUENCY_24MHZ:
            FL_RCC_RCHF_WriteTrimValue(RCHF24M_TRIM);
            PLL_Prescaler_1M = FL_RCC_PLL_PSC_DIV24;
            break;

        default:
            FL_RCC_RCHF_WriteTrimValue(RCHF8M_TRIM);
            PLL_Prescaler_1M = FL_RCC_PLL_PSC_DIV8;
            break;
    }

    FL_RCC_RCHF_SetFrequency(RCHFclock);
    FL_RCC_RCHF_Enable();
    return PLL_Prescaler_1M;
}


/**
  * @brief  ѡڲRCHFΪ໷Ĳοʱ
  *
  * @note   ໷ĲοʱҪԤƵ1MٽPLLƵ
  * @param  rchf  RCHFοʱѡ    This parameter can be one of the following values:    
  *           @arg @ref FL_RCC_RCHF_FREQUENCY_8MHZ
  *           @arg @ref FL_RCC_RCHF_FREQUENCY_16MHZ
  *           @arg @ref FL_RCC_RCHF_FREQUENCY_24MHZ
  *           @arg @ref FL_RCC_RCHF_FREQUENCY_32MHZ
  * @param  clock PLLƵƵ
  * @note   ʱƵΪ1M*clock+1Ƶʿɴ64M
  * @retval none
  */
uint32_t SelRCHFToPLL(const FL_RCHF_InitTypeDef *RCHFInitParameter,const FL_PLL_InitTypeDef *PLLInitParameter)
{
    uint32_t counter = 0;
    uint32_t readystatus = 0;

    /*pllʱӷΧΪ32M~64M򷵻쳣*/
    if((PLLInitParameter->Multiplier < 32U) || (PLLInitParameter->Multiplier > 64U))
    {
        return 1;
    }
    
    /* ʼRCHF */
    RCHFInit(RCHFInitParameter->Frequence);
    
    /* öȴΪ2CYCLE */
    FL_FLASH_SetReadWait(FLASH, FL_FLASH_READ_WAIT_2CYCLE);
    
    /* ϵͳʱӲ ʱԴ οʱԤƵϵ Ƶ */
    RCC_PLL_ConfigDomain_SYS(PLLInitParameter->ClockSource, PLLInitParameter->Prescaler, PLLInitParameter->Multiplier, FL_RCC_PLL_OUTPUT_X1);
    
    /* ʹPLL */
    FL_RCC_PLL_Enable();
    
    /* ȴPLLȶܽϵͳʱԴеPLLûн볬ʱ */
    do
    {
        readystatus = FL_RCC_IsActiveFlag_PLLReady();
        counter++;
        
    }
    while(( readystatus != 0x1U) && (counter != PLL_TIMEOUT));        
    
    /*  AHB APB1 APB2 ʱӷƵ */
    FL_RCC_SetAHBPrescaler(FL_RCC_AHBCLK_PSC_DIV1);
    FL_RCC_SetAPB1Prescaler(FL_RCC_APB1CLK_PSC_DIV1);
    FL_RCC_SetAPB2Prescaler(FL_RCC_APB2CLK_PSC_DIV1);
    
    /* ϵͳʱԴ */
    FL_RCC_SetSystemClockSource(FL_RCC_SYSTEM_CLK_SOURCE_PLL);

    /* ϵͳʱFLASHȴ */
    if(PLLInitParameter->Multiplier <= 24)
    {
        FL_FLASH_SetReadWait(FLASH, FL_FLASH_READ_WAIT_0CYCLE);
    }
    else if((PLLInitParameter->Multiplier > 24) && (PLLInitParameter->Multiplier <= 48))
    {
    FL_FLASH_SetReadWait(FLASH, FL_FLASH_READ_WAIT_1CYCLE);
    }
    else
    {
        FL_FLASH_SetReadWait(FLASH, FL_FLASH_READ_WAIT_2CYCLE);
    }

    SystemCoreClockUpdate();
    return 0;
}



/**
  * @brief  ⲿXTHFʼ
  * @param  Strength ǿȲ
  * @retval none
  */

void XTHFInit(uint32_t Strength)
{
    FL_GPIO_InitTypeDef      gpioInitStruction;
    FL_NVIC_ConfigTypeDef    InterruptConfigStruct;

    /* PC2.3óģ⹦ܣXTHF */
    gpioInitStruction.mode       = FL_GPIO_MODE_ANALOG;
    gpioInitStruction.outputType = FL_GPIO_OUTPUT_PUSHPULL;
    gpioInitStruction.pull       = FL_DISABLE;

    gpioInitStruction.pin = FL_GPIO_PIN_2 | FL_GPIO_PIN_3;
    FL_GPIO_Init(GPIOC, &gpioInitStruction);

    /* ǿ */
    FL_RCC_XTHF_WriteDriverStrength(Strength);
    FL_RCC_XTHF_Enable();
    FL_DelayMs(2);

    /* ʹͣж */
    FL_FDET_ClearFlag_XTHFFail();
    
    #if 1
    /* ʹܸƵⱨж */
    FL_FDET_EnableIT_XTHFFail();
    /* XTHFƵⱨж */
    InterruptConfigStruct.preemptPriority = 2;
    FL_NVIC_Init(&InterruptConfigStruct,HFDET_IRQn);
    #endif
}


/**
  * @brief  ѡⲿXTHFΪ໷Ĳοʱ
  *           
  * @param  clock PLLƵƵ
  * @note   ʱƵΪ1M*clock+1Ƶʿɴ64M
  * @retval none
  */
uint32_t SelXTHFToPLL(const FL_XTHF_InitTypeDef *XTHFInitParameter ,const FL_PLL_InitTypeDef *PLLInitParameter)
{
    uint32_t counter = 0;
    uint32_t readystatus = 0;
  
    /*pllʱӷΧΪ32M~64M򷵻쳣*/
    if((PLLInitParameter->Multiplier < 32U) || (PLLInitParameter->Multiplier > 64U))
    {
        return 1;
    }
    
    /* XTHFʼʹ */
    XTHFInit(XTHFInitParameter->DriverStrength);
    
    /* FLASHȴΪ2CYCLE */
    FL_FLASH_SetReadWait(FLASH, FL_FLASH_READ_WAIT_2CYCLE);
    
    /* ϵͳʱӲ ʱԴ οʱԤƵϵ Ƶ */
    RCC_PLL_ConfigDomain_SYS(PLLInitParameter->ClockSource, PLLInitParameter->Prescaler, PLLInitParameter->Multiplier, FL_RCC_PLL_OUTPUT_X1);
    
    /* ʹPLL */ 
    FL_RCC_PLL_Enable();

    /* ȴPLLȶܽϵͳʱԴеPLLûн볬ʱ */
    do
    {
        readystatus = FL_RCC_IsActiveFlag_PLLReady();
        counter++;
        
    }
    while(( readystatus != 0x1U) && (counter != PLL_TIMEOUT));        
    
    /*  AHB APB1 APB2 ʱӷƵ */
    FL_RCC_SetAHBPrescaler(FL_RCC_AHBCLK_PSC_DIV1);
    FL_RCC_SetAPB1Prescaler(FL_RCC_APB1CLK_PSC_DIV1);
    FL_RCC_SetAPB2Prescaler(FL_RCC_APB2CLK_PSC_DIV1);

    /* ϵͳʱԴ */
    FL_RCC_SetSystemClockSource(FL_RCC_SYSTEM_CLK_SOURCE_PLL);
    
    /* ϵͳʱFLASHȴ */
    if(PLLInitParameter->Multiplier <= 24)
    {
        FL_FLASH_SetReadWait(FLASH, FL_FLASH_READ_WAIT_0CYCLE);
    }
    else if((PLLInitParameter->Multiplier > 24) && (PLLInitParameter->Multiplier <= 48))
    {
        FL_FLASH_SetReadWait(FLASH, FL_FLASH_READ_WAIT_1CYCLE);
    }
    else
    {
        FL_FLASH_SetReadWait(FLASH, FL_FLASH_READ_WAIT_2CYCLE);
    }

    SystemCoreClockUpdate();
    return 0;
}



/**
  * @brief  лϵͳʱ
  *           
  * @param  *RCHFInitParameter RCHFãXTHFƥ
  * @param  *PLLInitParameter  PLL 
  * @retval none
  */
void SwtichToSenondarySystemClock(const FL_RCHF_InitTypeDef *RCHFInitParameter,
                                  const FL_PLL_InitTypeDef  *PLLInitParameter)
{
    uint32_t counter, PLL_Prescaler_1M;
    
    PLL_Prescaler_1M = RCHFInit(RCHFInitParameter->Frequence);

    /* ϵͳʱԴ */
    FL_RCC_SetSystemClockSource(FL_RCC_SYSTEM_CLK_SOURCE_RCHF);
  
    /* PLL */ 
    FL_RCC_PLL_Disable();
  
    /* ϵͳʱӲ ʱԴ οʱԤƵϵ Ƶ */
    RCC_PLL_ConfigDomain_SYS(FL_RCC_PLL_CLK_SOURCE_RCHF, PLL_Prescaler_1M, PLLInitParameter->Multiplier, FL_RCC_PLL_OUTPUT_X1);
  
    /* ʹPLL */ 
    FL_RCC_PLL_Enable();
  
    counter = PLL_TIMEOUT;
    /* ȴPLL */
    while((0x0 == FL_RCC_IsActiveFlag_PLLReady()) && (--counter));
    
    /* ֹƵⱨж */
    FL_FDET_DisableIT_XTHFFail();
    
    /* XTHFͣжϱʶ */
    FL_FDET_ClearFlag_XTHFFail();
    
    /* ϵͳʱԴ */
    FL_RCC_SetSystemClockSource(FL_RCC_SYSTEM_CLK_SOURCE_PLL);
  
    SYSCLKState = SYSCLK_ERROR;
    SystemCoreClockUpdate();
}


/**
  * @brief  лϵͳʱ
  *           
  * @param  *RCHFInitParameter RCHF
  * @param  *XTHFInitParameter XTHF
  * @param  *PLLInitParameter  PLL 
  * @retval none
  */
void SwtichToMainSystemClock(const FL_RCHF_InitTypeDef *RCHFInitParameter,
                             const FL_XTHF_InitTypeDef *XTHFInitParameter,
                             const FL_PLL_InitTypeDef  *PLLInitParameter)
{
    /* Ƿʱ쳣 */
    if ((SYSCLK_ERROR == SYSCLKState) &&\
       ((FL_RCC_SYSTEM_CLK_SOURCE_PLL != FL_RCC_GetSystemClockSource()) ||\
       (PLLInitParameter->ClockSource != FL_RCC_PLL_GetClockSource())))
    {
        /* ʹͣʧ¼XTHFĴӦ仯 */
        if (XTHF_REG_CHECK(XTHFInitParameter) || XTHF_REG_CHECK(XTHFInitParameter))
        {
            volatile uint32_t Reday[3];  /* Ż */
            uint32_t counter, primask;
          
            /* жXTHFȶ־Ƿ */
            Reday[0] = FL_RCC_XTHF_IsReady();
            Reday[1] = FL_RCC_XTHF_IsReady();
            Reday[2] = FL_RCC_XTHF_IsReady();
            if (Reday[0] && Reday[1] && Reday[2])
            {
                /* XTHFͣжϱʶ */
                FL_FDET_ClearFlag_XTHFFail();
                if(0x0 == FL_FDET_IsActiveFlag_XTHFFail())
                {
                    /* ׼лϵͳʱ */
                    (void)RCHFInit(RCHFInitParameter->Frequence);
                    /* ̬ */
                    primask = __get_PRIMASK();
                    __disable_irq();
                    /* ϵͳʱԴ */
                    FL_RCC_SetSystemClockSource(FL_RCC_SYSTEM_CLK_SOURCE_RCHF);
                    
                    /* PLL */ 
                    FL_RCC_PLL_Disable();
                  
                    /* ϵͳʱӲ ʱԴ οʱԤƵϵ Ƶ */
                    RCC_PLL_ConfigDomain_SYS(PLLInitParameter->ClockSource, PLLInitParameter->Prescaler, PLLInitParameter->Multiplier, FL_RCC_PLL_OUTPUT_X1);
                  
                    /* ʹPLL */
                    FL_RCC_PLL_Enable();
                  
                    /* ȴPLL */
                    counter = PLL_TIMEOUT;
                    do
                    {   /* жPLLס־ */
                        Reday[0] = FL_RCC_IsActiveFlag_PLLReady();
                        Reday[1] = FL_RCC_IsActiveFlag_PLLReady();
                        Reday[2] = FL_RCC_IsActiveFlag_PLLReady();
                        if (Reday[0] && Reday[1] && Reday[2])
                        {

                            if (FL_FDET_IsActiveFlag_XTHFFail()) break; 

                            SYSCLKState = SYSCLK_NORMAL;

                            FL_RCC_SetSystemClockSource(FL_RCC_SYSTEM_CLK_SOURCE_PLL);
                            SystemCoreClockUpdate();
                            break;
                        }
                    }while((0x0 == FL_FDET_IsActiveFlag_XTHFFail()) && (--counter));
                    
                    /* δɹлϵͳʱӣָϵͳʱ */
                    if (SYSCLK_NORMAL != SYSCLKState)
                    {
                        SwtichToSenondarySystemClock(&RCHF_InitParameter, &PLL_InitParameter);
                    }
                  
                    /* ȡ̬ */
                    __set_PRIMASK(primask);
                }
            }
        }
        else
        {
            (void)RCHFInit(RCHFInitParameter->Frequence);
            FL_RCC_SetSystemClockSource(FL_RCC_SYSTEM_CLK_SOURCE_RCHF);
            FL_RCC_PLL_Disable();
          
            /* ֹƵⱨж */
            FL_FDET_DisableIT_XTHFFail();
          
            SYSCLKState = SYSCLK_NORMAL;
            (void)SelXTHFToPLL(&XTHF_InitParameter, &PLL_InitParameter);
        }
    }
}







