/**    
  ******************************************************************************
  * @attention    
  * Copyright 2023 SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICRO.)
  *        
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met: 
  *    
  * 1. Redistributions of source code must retain the above copyright notice, 
  *    this list of conditions and the following disclaimer.
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software without
  *    specific prior written permission.
  *    
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS AS IS
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   
  * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
  * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT   
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
  *******************************************************************************
  */
#include "flash.h"
#include "svd.h"
#include <string.h>
#include "mf_config.h"
#include "verify.h"

/* USER Flashдʼַ */
#define USER_FLASH_PROG_ADDR    0x0000F000U

#define USER_FLASH_PAGE_MAX_NUM          8     /* ҳ Page */

#define BUFFER_SIZE     128
uint32_t DataBuffer[BUFFER_SIZE];

extern FL_ErrorStatus FL_FLASH_Write_Dma(FLASH_Type *FLASHx, uint32_t address, uint32_t *data);
extern FL_ErrorStatus FL_FLASH_Read_Dma(FLASH_Type *FLASHx, uint32_t address, uint32_t *data, uint16_t length);

/**
  * @brief  Flash дʾ
  * @param  void
  * @retval None
*/
void USER_Flash_RW(void)
{
    /* жMCUԴڲдʱ̷BOR縴λ */
    if (true == SVD_Result_Confirmed(SVD_HIGHER_THRESHOLD, 1U))
    {
        (void)FL_FLASH_PageErase(FLASH, USER_FLASH_PROG_ADDR);
        memset(DataBuffer, 0x55, BUFFER_SIZE * 4);
        (void)FL_FLASH_Program_Word(FLASH, USER_FLASH_PROG_ADDR, DataBuffer[0]);
    }
    
    /* жMCUԴڲдʱ̷BOR縴λ */
    if (true == SVD_Result_Confirmed(SVD_HIGHER_THRESHOLD, 1U))
    {
        (void)FL_FLASH_PageErase(FLASH, USER_FLASH_PROG_ADDR);
        memset(DataBuffer, 0xAA, BUFFER_SIZE * 4);
        (void)FL_FLASH_Program_Page(FLASH, USER_FLASH_PROG_ADDR / FL_FLASH_PAGE_SIZE_BYTE, DataBuffer);
    }
    
    /* жMCUԴڲдʱ̷BOR縴λ */ 
    if (true == SVD_Result_Confirmed(SVD_HIGHER_THRESHOLD, 1U))
    {
        (void)FL_FLASH_PageErase(FLASH, USER_FLASH_PROG_ADDR);
    }
}

/**
  * @brief  UserFlash д
  * @param  void
  * @retval None
*/
uint8_t Write_UserFlash(uint32_t Page, uint16_t offset, uint8_t *Buf, uint16_t Len)
{
    uint16_t h, i, k, status, result;
    static uint8_t BufPageBKUP[FL_FLASH_PAGE_SIZE_BYTE] __attribute__((aligned(4)));/* ضǰPage 4ֽڶ */
    uint8_t BufA[FL_FLASH_PAGE_SIZE_BYTE];
    uint32_t PageStartAddress;

    if (Page >= USER_FLASH_PAGE_MAX_NUM) return 1;  /* ַPAGE ҪԶ */
    
    h = Len;
    memcpy(BufA, Buf, Len);
    
    if (Page < USER_FLASH_CRC_PAGE)     /* CRCУݴŵҳ */
    {
        Crc_16Bits_Add2(BufA, Len);          /* crc */
        h = h + 2;                  /* ȼ2ֽcrc */
    }
    if ((offset + h) > FL_FLASH_PAGE_SIZE_BYTE) return 1; /* ռ */
    
    PageStartAddress = USER_FLASH_PROG_ADDR + Page * FL_FLASH_PAGE_SIZE_BYTE;
    
    /* жMCUԴڲдʱ̷BOR縴λ */
    if (true != SVD_Result_Confirmed(SVD_HIGHER_THRESHOLD, 1U))
    {
        return 2;
    }
    
    memcpy(BufPageBKUP, (void *)(PageStartAddress), FL_FLASH_PAGE_SIZE_BYTE);/* Ȼض */
    memcpy(&BufPageBKUP[offset], BufA, h);/* Ҫд벿 */

    for (i = 0; i < 2; i++) /* д2 */
    {
        for (k = 0; k < 3; k++) /* ÿд3 */
        {
            status = 0;
            result = FL_FLASH_PageErase(FLASH, USER_FLASH_PROG_ADDR + Page * FL_FLASH_PAGE_SIZE_BYTE);
            if (result != FL_PASS) status = 1;  /* ʧ */
            else
            {
                result = FL_FLASH_Program_Page(FLASH, (USER_FLASH_PROG_ADDR / FL_FLASH_PAGE_SIZE_BYTE)+Page , (uint32_t *)BufPageBKUP);
                if (result != FL_PASS)
                {
                    status = 1;
                }
            }
            
            if (status == 0)
            {
                break;
            }
        }
        if (status == 1) continue; /* ζ */

        /* --------------------------------- verify ------------------------------------- */
        result = memcmp(BufPageBKUP, (void *)(PageStartAddress), FL_FLASH_PAGE_SIZE_BYTE);
        if (result == 0)
        {
            if (Page < USER_FLASH_CRC_PAGE)/* CRCֵ */
            {
                memcpy(Buf, BufA, h);
            }
            return 0;
        }
    }
    return 3; /* fail */
}

/**
  * @brief  UserFlash 
  * @param  Page   ҳƫƵַ
            offset ҳƫƵַ
            Buf    Ŀ껺ַ
            Len    ݳ
  * @retval 0ȡɹ
            1ȡʧ
*/
uint8_t Read_UserFlash(uint32_t Page, uint16_t offset, uint8_t *Buf, uint16_t Len)
{
    uint16_t h;

    h = Len;
    if (Page < USER_FLASH_CRC_PAGE) h = h + 2;      /* ȼ2ֽcrc */

    if ((offset + h) > FL_FLASH_PAGE_SIZE_BYTE) return 1; /* ռ */
    
    memcpy(Buf, (void *)(USER_FLASH_PROG_ADDR + Page * FL_FLASH_PAGE_SIZE_BYTE + offset), h);
    return 0;
}

/**
  * @brief  UserFlash ݶ
  * @param  Page   ҳƫƵַ
            offset ҳƫƵַ
            Buf    Ŀ껺ַ
            Len    ݳ
  * @retval void
*/
void  ReadUserFlashWithBackup(uint32_t Page, uint16_t offset, uint8_t *Buf, uint16_t Len)
{
    if (Read_UserFlash(Page, offset, Buf, Len))
    {
        Read_UserFlash(Page + USER_FLASH_BKUP_PAGE, offset, Buf, Len);
    }
}

/**
  * @brief  UserFlash дд
  * @param  Page   ҳƫƵַ
            offset ҳƫƵַ
            Buf    Ŀ껺ַ
            Len    ݳ
  * @retval 0дɹ
            1ȡʧ
*/
uint8_t WriteUserFlashWithBackup(uint32_t Page, uint16_t offset, uint8_t *Buf, uint16_t Len)
{
    if (Write_UserFlash(Page, offset, Buf, Len)) return 1;

    Write_UserFlash(Page + USER_FLASH_BKUP_PAGE, offset, Buf, Len);

    return 0;
}


