/**    
  ******************************************************************************
  * @attention    
  * Copyright 2023 SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICRO.)
  *        
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met: 
  *    
  * 1. Redistributions of source code must retain the above copyright notice, 
  *    this list of conditions and the following disclaimer.
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software without
  *    specific prior written permission.
  *    
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS AS IS
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   
  * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
  * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT   
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
  *******************************************************************************
  */
#include "gptim.h"

/* ״̬ */
#define IS_CAPTURE          (1 << 0)
#define IS_CAPTURE_OVERFLOW (1 << 1)

/* ֵ */
static uint32_t  IC1Value = 0;
static uint32_t  IC2Value = 0;

static uint8_t   update_flag = 0;
static uint32_t  DutyCycle = 0;
static uint32_t  Frequency = 0;
static uint32_t  filter_count = 0;
static uint32_t  count = 0;


/**
  * @brief  GPTIM0 ж--벶
  * @param  void
  * @retval void
  */
void GPTIM0_IRQHandler(void)
{
    if((FL_GPTIM_IsEnabledIT_CC(GPTIM0, FL_GPTIM_CHANNEL_1)) \
      &&FL_GPTIM_IsActiveFlag_CC(GPTIM0, FL_GPTIM_CHANNEL_1))
    {
        /* 沶ֵ */
        count++;
        if(count == 2)
        {
            IC1Value = FL_GPTIM_ReadCompareCH1(GPTIM0);
            IC2Value = FL_GPTIM_ReadCompareCH2(GPTIM0);
            FL_GPTIM_Disable(GPTIM0);
            DutyCycle = 100 * (IC2Value+1) / (IC1Value+1);  /* ռձ */
            Frequency = (__SYSTEM_CLOCK / __SYSTEM_CLOCK_1MHz_PRE / (IC1Value+1));         /* Ƶ,Ƶ8M8Ƶ */
            count =0;
            FL_GPTIM_Enable(GPTIM0);
            
            filter_count++;
            if (filter_count >= 2)
            {
                filter_count = 0;
                update_flag = 1;                
            }
        }
        
        FL_GPTIM_ClearFlag_CC(GPTIM0, FL_GPTIM_CHANNEL_1);
    }
    
    if((FL_GPTIM_IsEnabledIT_CC(GPTIM0, FL_GPTIM_CHANNEL_2)) \
      &&FL_GPTIM_IsActiveFlag_CC(GPTIM0, FL_GPTIM_CHANNEL_2))
    {
        
        FL_GPTIM_ClearFlag_CC(GPTIM0, FL_GPTIM_CHANNEL_2);
    }

}



/**
  * @brief  GPTIM1 ʼ--PWM
  * @param  void
  * @retval void
  */
void GPTIM1_PWM_Init(void)
{
    FL_GPTIM_InitTypeDef        timInit = {0};
    FL_GPTIM_OC_InitTypeDef     timOCInit = {0};
    FL_GPIO_InitTypeDef         GPIOInit = {0};
    
    GPIOInit.pin          = FL_GPIO_PIN_5; /* ֹΪ FL_DISABLE */
    GPIOInit.mode         = FL_GPIO_MODE_DIGITAL;
    GPIOInit.outputType   = FL_GPIO_OUTPUT_PUSHPULL;
    GPIOInit.pull         = FL_DISABLE;
    GPIOInit.remapPin     = FL_DISABLE;
    FL_GPIO_Init(GPIOA, &GPIOInit);
    
    timInit.prescaler             = 64 - 1;                     /* Ƶϵ64 */
    timInit.counterMode           = FL_GPTIM_COUNTER_DIR_UP;      /* ϼ */
    timInit.autoReload            = 1000 - 1;                     /* Զװֵ1000 */
    timInit.clockDivision         = FL_GPTIM_CLK_DIVISION_DIV1;   /* ˲Ƶ */
    timInit.autoReloadState       = FL_ENABLE;                    /* Ԥװpreloadʹ */
    FL_GPTIM_Init(GPTIM1, &timInit);

    timOCInit.OCMode       = FL_GPTIM_OC_MODE_PWM1;               /* ȽģʽPWM1 */
    timOCInit.OCETRFStatus = FL_DISABLE;                          /* OC1REFETRӰ */
    timOCInit.OCFastMode   = FL_DISABLE;                          /* رտʹ */
    timOCInit.compareValue = 500;                                 /* Ƚֵ500 */
    timOCInit.OCPolarity   = FL_GPTIM_OC_POLARITY_NORMAL;         /* OC */
    timOCInit.OCPreload    = FL_DISABLE;                          /* OC preload Ч */
    FL_GPTIM_OC_Init(GPTIM1, FL_GPTIM_CHANNEL_4, &timOCInit);

    FL_GPTIM_Enable(GPTIM1);    /* ʹܶʱ */
}



/**
  * @brief  GPTIM0 ʼ--벶
  * @param  void
  * @retval void
  */
void GPTIM0_IC_Init(void)
{
    FL_GPIO_InitTypeDef        GPIO_InitStruct;
    FL_GPTIM_IC_InitTypeDef    GPTIM0_InitStruct;
    FL_GPTIM_SlaveInitTypeDef   Slave_InitStruct;
    FL_GPTIM_InitTypeDef       TimerBase_InitStruct;
    
    /* ʼŹ */
    GPIO_InitStruct.pin          = FL_GPIO_PIN_10;  /* ֹΪ FL_ENABLE */
    GPIO_InitStruct.mode         = FL_GPIO_MODE_DIGITAL;
    GPIO_InitStruct.outputType   = FL_GPIO_OUTPUT_PUSHPULL;
    GPIO_InitStruct.pull         = FL_DISABLE;
    GPIO_InitStruct.remapPin     = FL_ENABLE;
    FL_GPIO_Init(GPIOB, &GPIO_InitStruct);
    
    /* ʱƵʼ㣺ϵͳƵ/(Ƶϵ*Զװֵ) */
    TimerBase_InitStruct.prescaler       = __SYSTEM_CLOCK_1MHz_PRE - 1;
    TimerBase_InitStruct.counterMode     = FL_GPTIM_COUNTER_DIR_UP;
    TimerBase_InitStruct.autoReload      = GPTIM0_AUTORELOAD_VALUE-1;
    TimerBase_InitStruct.clockDivision   = FL_GPTIM_CLK_DIVISION_DIV1;
    TimerBase_InitStruct.autoReloadState = FL_ENABLE;
    FL_GPTIM_Init(GPTIM0, &TimerBase_InitStruct);
    
    /* ӻģʽ */
    Slave_InitStruct.ITRSourceGroup      = 0;                                     /* ڲԴã0 */
    Slave_InitStruct.slaveMode           = FL_GPTIM_SLAVE_MODE_TRGI_RISE_RST;     /* ģʽãλģʽ */
    Slave_InitStruct.triggerSrc          = FL_GPTIM_TIM_TS_TI1FP1;                /* ģʽԴãTI1FP1 */
    Slave_InitStruct.triggerDelay        = FL_DISABLE;                            /* ʱã */
    FL_GPTIM_SlaveMode_Init(GPTIM0,&Slave_InitStruct);
    
    /* 벶 CH1 */
    GPTIM0_InitStruct.ICPolarity         = FL_GPTIM_IC_POLARITY_NORMAL;
    GPTIM0_InitStruct.ICActiveInput      = FL_GPTIM_CHANNEL_MODE_INPUT_NORMAL;
    GPTIM0_InitStruct.ICPrescaler        = FL_GPTIM_IC_PSC_DIV1;
    GPTIM0_InitStruct.ICFilter           = FL_GPTIM_IC_FILTER_DIV1;
    GPTIM0_InitStruct.captureState       = FL_ENABLE;
    FL_GPTIM_IC_Init(GPTIM0, FL_GPTIM_CHANNEL_1, &GPTIM0_InitStruct);
    
    /* 벶 CH2 */
    GPTIM0_InitStruct.ICPolarity         = FL_GPTIM_IC_POLARITY_INVERT;           /* 벶ã½ز */
    GPTIM0_InitStruct.ICActiveInput      = FL_GPTIM_CHANNEL_MODE_INPUT_CROSSOVER; /* ͨģʽãCC1ͨΪ룬IC1ӳTI2 */
    FL_GPTIM_IC_Init(GPTIM0, FL_GPTIM_CHANNEL_2, &GPTIM0_InitStruct);
    
    /* ׽־ */
    FL_GPTIM_ClearFlag_CCOverflow(GPTIM0, FL_GPTIM_CHANNEL_1);
    /* CH1벶жϱ־ */
    FL_GPTIM_ClearFlag_CC(GPTIM0, FL_GPTIM_CHANNEL_1);
    /* ʹCH1벶ж */
    FL_GPTIM_EnableIT_CC(GPTIM0, FL_GPTIM_CHANNEL_1);
    
    /* CH2벶жϱ־ */
    FL_GPTIM_ClearFlag_CC(GPTIM0, FL_GPTIM_CHANNEL_2);
    /* ʹCH2벶ж */
    FL_GPTIM_EnableIT_CC(GPTIM0, FL_GPTIM_CHANNEL_2);
    
    /* NVIC */
    NVIC_DisableIRQ(GPTIM0_IRQn);
    NVIC_SetPriority(GPTIM0_IRQn, GPTIM0_IRQ_PRI );        /* жȼ */
    NVIC_EnableIRQ(GPTIM0_IRQn);
    
    /* ʹGPTIM0 */
    FL_GPTIM_Enable(GPTIM0);
}

/**
  * @brief  GPTIM0 Ƚ״̬
  * @param  void
  * @retval void
  */
void clear_pwm_update_flag(void)
{
    FL_GPTIM_Disable(GPTIM0);
    FL_GPTIM_ClearFlag_CC(GPTIM0, FL_GPTIM_CHANNEL_1);
    FL_GPTIM_ClearFlag_CC(GPTIM0, FL_GPTIM_CHANNEL_2);
    count = 0;
    filter_count = 0;
    update_flag = 0;
    FL_GPTIM_Enable(GPTIM0);
}


/**
  * @brief  ȡ GPTIM0 PWMռձȺƵ
  * @param  frequencyƵ
            dutyռձ
  * @retval 0ȡɹ
            1ȡʧ
  */
uint8_t get_pwm_status(uint32_t *frequency, uint8_t *duty)
{
    app_struct_type temp_app_struct;
    uint32_t primask;    
        
    /* жȫֱ̬ */
    primask = __get_PRIMASK();
    __disable_irq();
    memcpy((void *)&temp_app_struct, (void *)&g_app_struct, sizeof(app_struct_type));
    __set_PRIMASK(primask);    
    
    if (update_flag)
    {
        update_flag = 0;
        *frequency = Frequency;
        *duty = DutyCycle;        
        return 0;
    }
    else if (temp_app_struct.ctrl_data.str.pwm_duty == 0)
    {
        *frequency = 0;
        *duty = 0;        
        return 0; 
    }
    else if (temp_app_struct.ctrl_data.str.pwm_duty == 100)
    {
        *frequency = 0;
        *duty = 100;        
        return 0;
    }
    else
    {
        *frequency = 0xFFFFFF;/* Ƿֵ */
        *duty = 0xFF;
        return 1;/* Ƿֵ */
    }
}


/**
  * @brief   GPTIM1 PWMռձȺƵ
  * @param  frequencyƵ
            dutyռձ
  * @retval void
  */
void SET_PWM_Para(uint32_t frequency, uint8_t duty)
{
    FL_GPTIM_InitTypeDef        timInit = {0};
    FL_GPTIM_OC_InitTypeDef     timOCInit = {0};
    uint32_t temp_loader, temp_compare;
    
    if (((frequency < 25) || (frequency > 10000)) ||duty > 100)
    {
        return;
    }
    temp_loader= __SYSTEM_CLOCK / (__SYSTEM_CLOCK_1MHz_PRE * frequency);
    if (duty == 0)
    {
        timOCInit.OCPolarity     = FL_GPTIM_OC_POLARITY_INVERT;     /* ԣ */        
        temp_compare = 0;        
    }
    else if (duty == 100)
    {
        timOCInit.OCPolarity     = FL_GPTIM_OC_POLARITY_NORMAL;     /* ԣ */
        temp_compare = 0;
    }
    else
    {
        timOCInit.OCPolarity     = FL_GPTIM_OC_POLARITY_NORMAL;     /* ԣ */
        temp_compare = (duty) * temp_loader/100 ;
    }
    
    FL_GPTIM_Disable(GPTIM1);         /* ܶʱ */
    
    timInit.prescaler             = __SYSTEM_CLOCK_1MHz_PRE - 1;  /* ƵĿƵ */
    timInit.counterMode           = FL_GPTIM_COUNTER_DIR_UP;      /* ϼ */
    timInit.autoReload            = temp_loader - 1;
    timInit.clockDivision         = FL_GPTIM_CLK_DIVISION_DIV1;   /* ˲Ƶ */
    timInit.autoReloadState       = FL_ENABLE;                    /* Ԥװpreloadʹ */
    FL_GPTIM_Init(GPTIM1, &timInit);

    timOCInit.OCMode       = FL_GPTIM_OC_MODE_PWM1;               /* ȽģʽPWM1 */
    timOCInit.OCETRFStatus = FL_DISABLE;                          /* OC1REFETRӰ */
    timOCInit.OCFastMode   = FL_DISABLE;                          /* رտʹ */
    timOCInit.compareValue = temp_compare;                        /* Ƚֵ500 */
    timOCInit.OCPolarity   = FL_GPTIM_OC_POLARITY_NORMAL;         /* OC */
    timOCInit.OCPreload    = FL_DISABLE;                          /* OC preload Ч */
//    FL_GPTIM_OC_Init(GPTIM1, FL_GPTIM_CHANNEL_1, &timOCInit);
    
//    timOCInit.compareValue = temp_compare;                                 /* Ƚֵ500 */
//    FL_GPTIM_OC_Init(GPTIM1, FL_GPTIM_CHANNEL_2, &timOCInit);
    
//    timOCInit.compareValue = temp_compare;                                 /* Ƚֵ500 */
//    FL_GPTIM_OC_Init(GPTIM1, FL_GPTIM_CHANNEL_3, &timOCInit);
    
//    timOCInit.compareValue = temp_compare;                                 /* Ƚֵ500 */
    FL_GPTIM_OC_Init(GPTIM1, FL_GPTIM_CHANNEL_4, &timOCInit);

    FL_GPTIM_Enable(GPTIM1);    /* ʹܶʱ */
    
}



/**
  * @brief  GPTIM0ģ鸴λ
  * @param  void
  * @retval void
  */
void GPTIM0_DeInit(void)
{
    /* ָĴλֵ */
    FL_GPTIM_DeInit(GPTIM0);

    /* жϹ */
    NVIC_ClearPendingIRQ(GPTIM0_IRQn);
    
    /* رж */
    NVIC_DisableIRQ(GPTIM0_IRQn);
}


/**
  * @brief  GPTIM1ģ鸴λ
  * @param  void
  * @retval void
  */
void GPTIM1_DeInit(void)
{
    /* ָĴλֵ */
    FL_GPTIM_DeInit(GPTIM1);

    /* жϹ */
    NVIC_ClearPendingIRQ(GPTIM1_IRQn);
    
    /* رж */
    NVIC_DisableIRQ(GPTIM1_IRQn);
}



#if (SLEEP_SUPPORT == 1)

/**
  * @brief  GPTIM0 ǰʼ
  * @param  void
  * @retval void
  */
void GPTIM0_Sleep_Init(void)
{
    /* رշǻ */
    FL_GPTIM_Disable(GPTIM0);  /* ܶʱ */
    FL_GPTIM_DisableIT_CC(GPTIM0, FL_GPTIM_CHANNEL_1);
    FL_GPTIM_DisableIT_CC(GPTIM0, FL_GPTIM_CHANNEL_2);
    
    /* ׽־ */
    FL_GPTIM_ClearFlag_CCOverflow(GPTIM0, FL_GPTIM_CHANNEL_1);
    FL_GPTIM_ClearFlag_CCOverflow(GPTIM0, FL_GPTIM_CHANNEL_2);
    
    /* CH1벶жϱ־ */
    FL_GPTIM_ClearFlag_CC(GPTIM0, FL_GPTIM_CHANNEL_1);
    /* CH2벶жϱ־ */
    FL_GPTIM_ClearFlag_CC(GPTIM0, FL_GPTIM_CHANNEL_2);
    
    Close_And_ClearIRQ(GPTIM0_IRQn);
    
    /* رշǻIO */
    CloseIO(GPIOB, FL_GPIO_PIN_10);
    
    /* Դ */
}

/**
  * @brief  GPTIM0 Ѻʼ
  * @param  0-ͨ 1-ΪλѵԴ
  * @retval void
  */
void GPTIM0_Awake_Init(uint8_t awake_type)
{
    /* 򿪷ǻIO */
    AltFunIO( GPIOB, FL_GPIO_PIN_10, FL_GPIO_OUTPUT_PUSHPULL, FL_DISABLE, FL_ENABLE ); /* PB10 --> GPTIM벶 */  
    
    /* ʼ */
    update_flag = 0;
    IC1Value = 0;
    IC2Value = 0;
    DutyCycle = 0;
    Frequency = 0;
    filter_count = 0;
    count = 0;
    
    /* Դ */
    
    /* ǻ账 */
    
    /* ׽־ */
    FL_GPTIM_ClearFlag_CCOverflow(GPTIM0, FL_GPTIM_CHANNEL_1);
    FL_GPTIM_ClearFlag_CCOverflow(GPTIM0, FL_GPTIM_CHANNEL_2);
    
    /* CH1벶жϱ־ */
    FL_GPTIM_ClearFlag_CC(GPTIM0, FL_GPTIM_CHANNEL_1);
    /* CH2벶жϱ־ */
    FL_GPTIM_ClearFlag_CC(GPTIM0, FL_GPTIM_CHANNEL_2);
    
    FL_GPTIM_EnableIT_CC(GPTIM0, FL_GPTIM_CHANNEL_1);
    FL_GPTIM_EnableIT_CC(GPTIM0, FL_GPTIM_CHANNEL_2);
    
    NVIC_EnableIRQ(GPTIM0_IRQn);
    FL_GPTIM_Enable(GPTIM0);  /* ʹܶʱ */    
}

/**
  * @brief  GPTIM0 sleepж
  * @param  void
  * @retval 1: Sleep
  *         0: Wake
  */
uint8_t GPTIM0_IS_Sleep(void)
{
    if (1)/* ʱɽsleep */
    {
        return 1;
    }
    else
    {
        return 0;
    }
}

/**
  * @brief  GPTIM1 ǰʼ
  * @param  void
  * @retval void
  */
void GPTIM1_Sleep_Init(void)
{
    /* رշǻ */
    FL_GPTIM_Disable(GPTIM1);  /* ܶʱ */
    
    /* رշǻIO */
//    CloseIO(GPIOC, FL_GPIO_PIN_0);
//    CloseIO(GPIOC, FL_GPIO_PIN_1);
//    CloseIO(GPIOA, FL_GPIO_PIN_4);
    CloseIO(GPIOA, FL_GPIO_PIN_5);
    
    /* Դ */
}

/**
  * @brief  GPTIM1 Ѻʼ
  * @param  0-ͨ 1-ΪλѵԴ
  * @retval void
  */
void GPTIM1_Awake_Init(uint8_t awake_type)
{
    /* 򿪷ǻIO */
//    AltFunIO( GPIOC, FL_GPIO_PIN_0, FL_GPIO_OUTPUT_PUSHPULL, FL_DISABLE, FL_DISABLE ); /* PC0 --> GPTIM1 ȽPWM */
//    AltFunIO( GPIOC, FL_GPIO_PIN_1, FL_GPIO_OUTPUT_PUSHPULL, FL_DISABLE, FL_DISABLE ); /* PC1 --> GPTIM1 ȽPWM */
//    AltFunIO( GPIOA, FL_GPIO_PIN_4, FL_GPIO_OUTPUT_PUSHPULL, FL_DISABLE, FL_DISABLE ); /* PA4 --> GPTIM1 ȽPWM */
    AltFunIO( GPIOA, FL_GPIO_PIN_5, FL_GPIO_OUTPUT_PUSHPULL, FL_DISABLE, FL_DISABLE ); /* PA5 --> GPTIM1 ȽPWM */
    
    /* Դ */
    
    /* ǻ账 */
    FL_GPTIM_Enable(GPTIM1);  /* ʹܶʱ */    
}

/**
  * @brief  GPTIM1 sleepж
  * @param  void
  * @retval 1: Sleep
  *         0: Wake
  */
uint8_t GPTIM1_IS_Sleep(void)
{
    if (1)/* ʱɽsleep */
    {
        return 1;
    }
    else
    {
        return 0;
    }
}

#endif

