/**    
  ******************************************************************************
  * @attention    
  * Copyright 2023 SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICRO.)
  *        
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met: 
  *    
  * 1. Redistributions of source code must retain the above copyright notice, 
  *    this list of conditions and the following disclaimer.
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software without
  *    specific prior written permission.
  *    
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS AS IS
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   
  * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
  * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT   
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
  *******************************************************************************
  */
#include "svd_interrupt.h"


/* SVD */
static volatile SVD_RESULT eSVDResult = SVD_HIGHER_THRESHOLD; 



/**
  * @brief  SVD жϷ
  * @param  void
  * @retval void
  */
void SVD_IRQHandler(void)
{
    if(FL_SET == FL_SVD_IsActiveFlag_PowerFall(SVD))        /* Ƿѹж */
    {
        FL_SVD_ClearFlag_PowerFall(SVD);
        eSVDResult = SVD_BELOW_THRESHOLD;;
    }
    else if(FL_SET == FL_SVD_IsActiveFlag_PowerRise(SVD))   /* Ƿѹָж */
    {
        FL_SVD_ClearFlag_PowerRise(SVD);
        eSVDResult = SVD_HIGHER_THRESHOLD;
    }
}


/**
  * @brief  SVD жϼʼ
  * @param  void
  * @retval void
  */
void SVD_Interrupt_Init(FL_FunState DigitalFilter)
{
    FL_SVD_InitTypeDef    SVD_InitStruct;

    SVD_InitStruct.workMode         = FL_SVD_WORK_MODE_CONTINUOUS;     /* SVDʹ */
    SVD_InitStruct.enablePeriod     = FL_SVD_ENABLE_PERIOD_1000MS;     /* ЪʹµļЪʱ */
    SVD_InitStruct.referenceVoltage = FL_SVD_REFERENCE_1P0V;           /* ο׼ѹ */
    SVD_InitStruct.warningThreshold = FL_SVD_WARNING_THRESHOLD_GROUP8; /* ֵλ */
    SVD_InitStruct.digitalFilter    = FL_ENABLE;                       /* ˲ */
    SVD_InitStruct.SVSChannel       = FL_DISABLE;                      /* SVSͨ */
    FL_SVD_Init(SVD, &SVD_InitStruct);
    
    /* ѹж */
    FL_SVD_ClearFlag_PowerFall(SVD);
    FL_SVD_EnableIT_PowerFall(SVD);

    /* Ƿѹж */
    FL_SVD_ClearFlag_PowerRise(SVD);
    FL_SVD_EnableIT_PowerRise(SVD);
    
    /* жʹ */
    NVIC_DisableIRQ(SVD_IRQn);
    NVIC_SetPriority(SVD_IRQn, 2);
    NVIC_EnableIRQ(SVD_IRQn);
    
    /* ʹSVD */
    FL_SVD_Enable(SVD);
    
    /* ע⣺ʹ˲ӦʹSVDRѯ δʹ˲ӦʹSVDOѯ */
    if (FL_ENABLE == DigitalFilter)
    {
        eSVDResult = SVD_SVDR_POLL();
    }
    else
    {
        eSVDResult = SVD_SVDO_POLL();
    }
}


/**
  * @brief  SVDж
  * @param  void
  * @retval void
  */
void SVD_POLL(void)
{
    
    /* SVDڲ˲ĵѹ־ʹ˲ʱ壻д˼Ĵ */
    if(FL_RESET == FL_SVD_GetLatchedPowerStatus(SVD))  
    {
        /* SVD⵽Ƿѹ */
        /* ...          */
    }
}



/**
  * @brief  SVDж
  * @param  void
  * @retval None
  */
SVD_RESULT SVD_IT(void)
{
    /* ࣬쳣縴λ */
    if (eSVDResult > SVD_HIGHER_THRESHOLD)
    {
        eSVDResult = SVD_BELOW_THRESHOLD; 
    } 
    if(eSVDResult == SVD_BELOW_THRESHOLD)
    {
        /* SVD⵽Ƿѹ */ 
    }
    else
    {
        /* SVD⵽Ƿѹָ */ 
    }
    /* ӦõƷʱע */
    return eSVDResult;
   
}

/**
  * @brief  SVD DeInitialization function
  * @param  void
  * @retval None
  */
void SVD_Interrupt_DeInit(void)
{
    /* ʹSVDǷѹ͹ѹж */
    FL_SVD_ClearFlag_PowerRise(SVD);
    FL_SVD_ClearFlag_PowerFall(SVD);
    FL_SVD_DisableIT_PowerRise(SVD );
    FL_SVD_DisableIT_PowerFall(SVD );
    /* ֹNVICж */
    NVIC_ClearPendingIRQ(SVD_IRQn);
    NVIC_DisableIRQ(SVD_IRQn);
}

 /**
  * @brief  SVDģ鸴λ
  * @param  void
  * @retval void
  */
void SVD_DeInit(void)
{
    /* ָĴλֵ */
    FL_SVD_DeInit(SVD);

    /* жϹ */
    NVIC_ClearPendingIRQ(SVD_IRQn);
    
    /* رж */
    NVIC_DisableIRQ(SVD_IRQn);
}




