/**    
  ******************************************************************************
  * @attention    
  * Copyright 2023 SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICRO.)
  *        
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met: 
  *    
  * 1. Redistributions of source code must retain the above copyright notice, 
  *    this list of conditions and the following disclaimer.
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software without
  *    specific prior written permission.
  *    
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS AS IS
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   
  * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
  * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT   
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
  *******************************************************************************
  */
#include "main.h"
#include "uart.h"
#include "lin.h"

struct UARTOpStruct UARTxOp;

/**
  * @brief  UARTʼ
  * @param  UARTx: ѡ
  * @retval void
  */
void Uartx_Init(UART_Type *UARTx)
{
    FL_GPIO_InitTypeDef GPIO_InitStruct = {0};
    FL_UART_InitTypeDef UART_InitStruct = {0};

    switch((uint32_t)UARTx)
    {
        case UART0_BASE:

            /* PA2:UART0-RX */
            GPIO_InitStruct.pin         = FL_GPIO_PIN_2;
            GPIO_InitStruct.mode        = FL_GPIO_MODE_DIGITAL;
            GPIO_InitStruct.outputType  = FL_GPIO_OUTPUT_PUSHPULL;
            GPIO_InitStruct.pull        = FL_ENABLE;
            GPIO_InitStruct.remapPin    = FL_DISABLE;
            FL_GPIO_Init(GPIOA, &GPIO_InitStruct);
            
            /* PA3:UART0-TX */
            GPIO_InitStruct.pin         = FL_GPIO_PIN_3;
            GPIO_InitStruct.mode        = FL_GPIO_MODE_DIGITAL;
            GPIO_InitStruct.outputType  = FL_GPIO_OUTPUT_PUSHPULL;
            GPIO_InitStruct.pull        = FL_DISABLE;
            GPIO_InitStruct.remapPin    = FL_DISABLE;
            FL_GPIO_Init(GPIOA, &GPIO_InitStruct);
        
            UART_InitStruct.clockSrc            = FL_RCC_UART0_CLK_SOURCE_APB1CLK;          /* ʱԴ */
            /* NVICж */
            NVIC_DisableIRQ(UART0_IRQn);
            NVIC_SetPriority(UART0_IRQn, 2); /* жȼ */
            NVIC_EnableIRQ(UART0_IRQn);
            break;

        case UART1_BASE:

            /* PC2:UART1-RX */
            GPIO_InitStruct.pin        = FL_GPIO_PIN_2;
            GPIO_InitStruct.mode       = FL_GPIO_MODE_DIGITAL;
            GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
            GPIO_InitStruct.pull       = FL_ENABLE;
            GPIO_InitStruct.remapPin   = FL_DISABLE;
            FL_GPIO_Init(GPIOC, &GPIO_InitStruct);
            
            /* PC3:UART1-TX */
            GPIO_InitStruct.pin        = FL_GPIO_PIN_3;
            GPIO_InitStruct.mode       = FL_GPIO_MODE_DIGITAL;
            GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
            GPIO_InitStruct.pull       = FL_DISABLE;
            GPIO_InitStruct.remapPin   = FL_DISABLE;
            FL_GPIO_Init(GPIOC, &GPIO_InitStruct);
        
            UART_InitStruct.clockSrc            = FL_RCC_UART1_CLK_SOURCE_APB1CLK;          /* ʱԴ */

            /* NVICж */
            NVIC_DisableIRQ(UART1_IRQn);
            NVIC_SetPriority(UART1_IRQn, 2); /* жȼ */
            NVIC_EnableIRQ(UART1_IRQn);
            break;

        case UART2_BASE:
            /* PB1:UART2-TX */
            GPIO_InitStruct.pin         = FL_GPIO_PIN_1;
            GPIO_InitStruct.mode        = FL_GPIO_MODE_DIGITAL;
            GPIO_InitStruct.outputType  = FL_GPIO_OUTPUT_PUSHPULL;
            GPIO_InitStruct.pull        = FL_DISABLE;
            GPIO_InitStruct.remapPin    = FL_ENABLE;
            FL_GPIO_Init(GPIOB, &GPIO_InitStruct);            
        
            /* PB0:UART2-RX */
            GPIO_InitStruct.pin         = FL_GPIO_PIN_0;
            GPIO_InitStruct.mode        = FL_GPIO_MODE_DIGITAL;
            GPIO_InitStruct.outputType  = FL_GPIO_OUTPUT_PUSHPULL;
            GPIO_InitStruct.pull        = FL_ENABLE;
            GPIO_InitStruct.remapPin    = FL_ENABLE;
            FL_GPIO_Init(GPIOB, &GPIO_InitStruct);

            /* NVICж */
            NVIC_DisableIRQ(UART2_IRQn);
            NVIC_SetPriority(UART2_IRQn, LIN_UART_IRQ_PRI); /* жȼ */
            NVIC_EnableIRQ(UART2_IRQn);
            break;


        case UART4_BASE:
            /* PB2:UART4-RX */
            GPIO_InitStruct.pin          = FL_GPIO_PIN_2;
            GPIO_InitStruct.mode         = FL_GPIO_MODE_DIGITAL;
            GPIO_InitStruct.outputType   = FL_GPIO_OUTPUT_PUSHPULL;
            GPIO_InitStruct.pull         = FL_ENABLE;
            GPIO_InitStruct.remapPin     = FL_DISABLE;
            FL_GPIO_Init(GPIOB, &GPIO_InitStruct);
        
            /* PB3:UART4-TX */
            GPIO_InitStruct.pin          = FL_GPIO_PIN_3;
            GPIO_InitStruct.mode         = FL_GPIO_MODE_DIGITAL;
            GPIO_InitStruct.outputType   = FL_GPIO_OUTPUT_PUSHPULL;
            GPIO_InitStruct.pull         = FL_DISABLE;
            GPIO_InitStruct.remapPin     = FL_DISABLE;
            FL_GPIO_Init(GPIOB, &GPIO_InitStruct);


            /* NVICж */
            NVIC_DisableIRQ(UART4_IRQn);
            NVIC_SetPriority(UART4_IRQn, 2); /* жȼ */
            NVIC_EnableIRQ(UART4_IRQn);
            break;

        case UART5_BASE:
            /* PD0:UART5-RX */
            GPIO_InitStruct.pin            = FL_GPIO_PIN_0;
            GPIO_InitStruct.mode           = FL_GPIO_MODE_DIGITAL;
            GPIO_InitStruct.outputType     = FL_GPIO_OUTPUT_PUSHPULL;
            GPIO_InitStruct.pull           = FL_ENABLE;
            GPIO_InitStruct.remapPin       = FL_DISABLE;
            FL_GPIO_Init(GPIOD, &GPIO_InitStruct);
        
            /* PD1:UART5-TX */
            GPIO_InitStruct.pin            = FL_GPIO_PIN_1;
            GPIO_InitStruct.mode           = FL_GPIO_MODE_DIGITAL;
            GPIO_InitStruct.outputType     = FL_GPIO_OUTPUT_PUSHPULL;
            GPIO_InitStruct.pull           = FL_DISABLE;
            GPIO_InitStruct.remapPin       = FL_DISABLE;
            FL_GPIO_Init(GPIOD, &GPIO_InitStruct);


            /* NVICж */
            NVIC_DisableIRQ(UART5_IRQn);
            NVIC_SetPriority(UART5_IRQn, 2); /* жȼ */
            NVIC_EnableIRQ(UART5_IRQn);
            break;

        default:
            break;
    }
    
    UART_InitStruct.baudRate            = LIN_BAUDRATE;                             /*  */
    UART_InitStruct.dataWidth           = FL_UART_DATA_WIDTH_8B;                    /* λ */
    UART_InitStruct.stopBits            = FL_UART_STOP_BIT_WIDTH_1B;                /* ֹͣλ */
    UART_InitStruct.parity              = FL_UART_PARITY_NONE;                      /* żУ */
    UART_InitStruct.transferDirection   = FL_UART_DIRECTION_TX;                  /* -ʹ */
    
    FL_UART_Init(UARTx, &UART_InitStruct);
    
}




