/**    
  ******************************************************************************
  * @attention    
  * Copyright 2023 SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICRO.)
  *        
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met: 
  *    
  * 1. Redistributions of source code must retain the above copyright notice, 
  *    this list of conditions and the following disclaimer.
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software without
  *    specific prior written permission.
  *    
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS AS IS
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   
  * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
  * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT   
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
  *******************************************************************************
  */

#include <string.h>
#include "mf_config.h"
#include "fm33le0xxa_fl.h"
#include "i2c.h"
#include "flash.h"
#include "lin.h"
#include "verify.h"
#include "led.h"
#include "clock.h"


/**
  * @brief  УĴ
  * @param  RegAddrĴַ
  *         ValueУֵ
  * @retval 1 Уͨ
  *         0 У鲻ͨ
  */
uint8_t CheckSysReg( __IO uint32_t *RegAddr, uint32_t Value )
{
    /* Ĵֵͬ¸ֵ */
    if( *RegAddr != Value ) 
    {
        *RegAddr = Value;
        return 1;
    }
    else
    {
        return 0;
    }
}


/**
  * @brief  ѯNVICĴӦжǷ
  * @param  IRQnж
  * @retval 1 
  *         0 ر
  */
uint8_t CheckNvicIrqEn( IRQn_Type IRQn )
{
    if( 0 == ( NVIC->ISER[0U] & ((uint32_t)(1UL << (((uint32_t)(int32_t)IRQn) & 0x1FUL)))) )
        return 0;
    else
        return 1;
}


/**
  * @brief  رղʹúδʹõIO
  * @param  void
  * @retval void
  */
void Close_None_GPIO(void)
{
    /* رоƬʹõIO(޶Ӧ) */
    CloseIO( GPIOA, FL_GPIO_PIN_0 );
    CloseIO( GPIOA, FL_GPIO_PIN_1 );
    CloseIO( GPIOA, FL_GPIO_PIN_2 );
    CloseIO( GPIOA, FL_GPIO_PIN_3 );
    CloseIO( GPIOA, FL_GPIO_PIN_6 );
    CloseIO( GPIOA, FL_GPIO_PIN_7 );
    CloseIO( GPIOA, FL_GPIO_PIN_11 );
    CloseIO( GPIOA, FL_GPIO_PIN_12 );
    CloseIO( GPIOA, FL_GPIO_PIN_15 );
    
    CloseIO( GPIOB, FL_GPIO_PIN_2 );
    CloseIO( GPIOB, FL_GPIO_PIN_3 );
    CloseIO( GPIOB, FL_GPIO_PIN_5 );
    CloseIO( GPIOB, FL_GPIO_PIN_6 );
    CloseIO( GPIOB, FL_GPIO_PIN_7 );
    CloseIO( GPIOB, FL_GPIO_PIN_8 );
    CloseIO( GPIOB, FL_GPIO_PIN_12 );
    CloseIO( GPIOB, FL_GPIO_PIN_13 );
    CloseIO( GPIOB, FL_GPIO_PIN_14 );
    CloseIO( GPIOB, FL_GPIO_PIN_15 );
    
    CloseIO( GPIOC, FL_GPIO_PIN_11 );
    CloseIO( GPIOC, FL_GPIO_PIN_12 );
    CloseIO( GPIOC, FL_GPIO_PIN_13 );
    CloseIO( GPIOC, FL_GPIO_PIN_14 );
    CloseIO( GPIOC, FL_GPIO_PIN_15 );
    
    CloseIO( GPIOD, FL_GPIO_PIN_3 );
    CloseIO( GPIOD, FL_GPIO_PIN_4 );
    CloseIO( GPIOD, FL_GPIO_PIN_5 );
    CloseIO( GPIOD, FL_GPIO_PIN_6 );
    CloseIO( GPIOD, FL_GPIO_PIN_9 );
    CloseIO( GPIOD, FL_GPIO_PIN_10 );
    CloseIO( GPIOD, FL_GPIO_PIN_11 );
    CloseIO( GPIOD, FL_GPIO_PIN_12 );
    CloseIO( GPIOD, FL_GPIO_PIN_13 );
    CloseIO( GPIOD, FL_GPIO_PIN_14 );
    CloseIO( GPIOD, FL_GPIO_PIN_15 );
    
    
    /* رоƬδʹõIO(жӦţӦδʹ) */
    CloseIO( GPIOA, FL_GPIO_PIN_13 );
    CloseIO( GPIOA, FL_GPIO_PIN_8 );
    
    CloseIO( GPIOB, FL_GPIO_PIN_4 );
    CloseIO( GPIOB, FL_GPIO_PIN_9 );
    
}


/**
  * @brief  Ӧ蹦ż
  * @param  void
  * @retval void
  */
void Veri_Pad( void )
{    
    /* GPIO жϼĴУ */
    CheckSysReg( &GPIO->EXTIEDS , 0xFFFFFFFF );
    CheckSysReg( &GPIO->EXTIDF  , 0x00000010 );
    CheckSysReg( &GPIO->FOUTSEL , 0x00000000 );    /* FOUT1PB11Ϊ */
    CheckSysReg( &GPIO->PINWKEN , 0x00000000 );
    
    /********************************** GPIO  **********************************/
    /* LinSlave-UART2 */
    AltFunIO( GPIOB, FL_GPIO_PIN_1, FL_GPIO_OUTPUT_PUSHPULL, FL_DISABLE, FL_ENABLE );    /* PB1 --> LinSlave UART2 Tx */
    /* LinSlave-SLP */
    OutputIO( GPIOA, FL_GPIO_PIN_9,  FL_GPIO_OUTPUT_PUSHPULL );    /* PE9 --> LIN SLPſ */
    
    /*  PB11 */
    InputIO ( GPIOB, FL_GPIO_PIN_11,  FL_ENABLE );    /* PB11 -->  */    
    
    /* GPTIM0 벶 */
    AltFunIO( GPIOB, FL_GPIO_PIN_10, FL_GPIO_OUTPUT_PUSHPULL, FL_DISABLE, FL_ENABLE ); /* PB10 --> GPTIM0_CH1 벶 */
    
    /* GPTIM1 ȽPWM */
    AltFunIO( GPIOA, FL_GPIO_PIN_5, FL_GPIO_OUTPUT_PUSHPULL, FL_DISABLE, FL_DISABLE ); /* PA5 --> GPTIM1_CH4 ȽPWM */
    
    /* LED ͨGPIOģʽʹ */
    #if LED_TOG_MODE
    OutputIO( GPIOC, FL_GPIO_PIN_0, FL_GPIO_OUTPUT_PUSHPULL );    /* PC0 --> LED1ָʾ */
    OutputIO( GPIOC, FL_GPIO_PIN_1, FL_GPIO_OUTPUT_PUSHPULL );    /* PC1 --> LED2ָʾ */
    OutputIO( GPIOA, FL_GPIO_PIN_4, FL_GPIO_OUTPUT_PUSHPULL );    /* PA4 --> LED3ָʾ */
    #endif
    
    /* ⲿƵ XTHF */
    AnalogIO( GPIOC, FL_GPIO_PIN_2 );    /* PC2 --> XTHIN */
    AnalogIO( GPIOC, FL_GPIO_PIN_3 );    /* PC3 --> XTHOUT */
    
    /* SPI2 */
    AltFunIO( GPIOC, FL_GPIO_PIN_7,  FL_GPIO_OUTPUT_PUSHPULL, FL_DISABLE, FL_DISABLE );     /* PC7 --> SPI2_SCK */
    AltFunIO( GPIOC, FL_GPIO_PIN_8,  FL_GPIO_OUTPUT_PUSHPULL, FL_DISABLE, FL_DISABLE );     /* PC8 --> SPI2_MISO */
    AltFunIO( GPIOC, FL_GPIO_PIN_9,  FL_GPIO_OUTPUT_PUSHPULL, FL_DISABLE, FL_DISABLE );     /* PC9 --> SPI2_MOSI */
    AltFunIO( GPIOC, FL_GPIO_PIN_10, FL_GPIO_OUTPUT_PUSHPULL, FL_DISABLE, FL_DISABLE );     /* PC10 --> SPI2_SSN */   
    
    /* ADC */
    AnalogIO( GPIOD, FL_GPIO_PIN_0 );    /* PD0 --> ADC CH3 */
    AnalogIO( GPIOD, FL_GPIO_PIN_1 );    /* PD1 --> ADC CH4 */
    AnalogIO( GPIOD, FL_GPIO_PIN_2 );    /* PD2 --> ADC CH5 */
    AnalogIO( GPIOA, FL_GPIO_PIN_14 );   /* PA14 --> ADC CH7 */
    AnalogIO( GPIOC, FL_GPIO_PIN_6 );    /* PC6 --> /ADC CH11 */
    
    /* I2C */
    AltFunIO( GPIOC, FL_GPIO_PIN_4,  FL_GPIO_OUTPUT_PUSHPULL, FL_DISABLE, FL_ENABLE );    /* PC4 --> I2C SDA */
    AltFunIO( GPIOC, FL_GPIO_PIN_5,  FL_GPIO_OUTPUT_PUSHPULL, FL_DISABLE, FL_ENABLE );    /* PC5 --> I2C SCL */
    
    /* SWD */
    AltFunIO( GPIOD, FL_GPIO_PIN_7,  FL_GPIO_OUTPUT_PUSHPULL, FL_DISABLE, FL_DISABLE );    /* PD7 --> SWCLK */  
    AltFunIO( GPIOD, FL_GPIO_PIN_8,  FL_GPIO_OUTPUT_PUSHPULL, FL_DISABLE, FL_DISABLE );    /* PD8 --> SWIO */
    
}


/**
  * @brief  ʱУ
  * @param  void
  * @retval void
  */
void Veri_CMU( void )               
{
    uint32_t counter = 0;
    uint32_t readystatus = 0;    
    
    /* ʱѡ */
    CheckSysReg( &RCC->SYSCLKCR,  0x02008002 );
    CheckSysReg( &RCC->RCHFCR,  0x00000001 );
    if ((RCC->PLLCR & (0x00000033 | (PLLDB_VALUE << 16))) != (0x00000033 | (PLLDB_VALUE << 16)))
    {
        RCC->PLLCR = (0x00000033 | (PLLDB_VALUE << 16));
        
        do
        {
            readystatus = FL_RCC_IsActiveFlag_PLLReady();
            counter++;
            
            if ((counter % 100) == 0)
            {   /* 幷 */
                FL_IWDT_ReloadCounter(IWDT);  
            }
        } while((readystatus != 0x1U) && (counter != 0xFFFFFFFFU));  /* ȴPLL */
        
    }
    CheckSysReg( &RCC->XTLFCR,      0x00070000 );       
    CheckSysReg( &RCC->LSCLKSEL,    0x00000001 );     
    CheckSysReg( &RCC->XTHFCR,      0x00070701 );       
    CheckSysReg( &RCC->PCLKCR1,     0x800000F9 );      
    CheckSysReg( &RCC->PCLKCR2,     0x00000190 );      
    CheckSysReg( &RCC->PCLKCR3,     0x01000402 );      
    CheckSysReg( &RCC->PCLKCR4,     0x0000001D );      
    CheckSysReg( &RCC->OPCCR1,      0x00118400 );       
    CheckSysReg( &RCC->OPCCR2,      0x03111010 );       
    CheckSysReg( &RCC->AHBMCR,      0x80000000 );       
    
    #ifdef __DEBUG
    CheckSysReg( &DBG->CR,      0x00000003 ); /* ʱжʱ, ģʽ²رHCLKFCLK */
    #else
    CheckSysReg( &DBG->CR,      0x00000000 );
    #endif
}


/**
  * @brief  FLASH У
  * @param  void
  * @retval void
  */
void Veri_FLASH( void )
{
    CheckSysReg( &FLASH->RDCR,  0x00000002 );
}


/**
  * @brief  PMU У
  * @param  void
  * @retval void
  */
void Veri_PMU( void )
{
    CheckSysReg( &PMU->CR,      0x00020000 );
    CheckSysReg( &PMU->WKTR,    0xC0000001 );
    CheckSysReg( &PMU->IER,     0x00000014 );
}


/**
  * @brief  RMU У
  * @param  void
  * @retval void
  */
void Veri_RMU( void )
{
    CheckSysReg( &RMU->PDRCR,   0x00000001 );
    CheckSysReg( &RMU->BORCR,   0x0000000C );
}


/**
  * @brief  IWDT У
  * @param  void
  * @retval void
  */
void Veri_IWDT( void )
{
    CheckSysReg( &IWDT->IER,    0x00000000 );
    CheckSysReg( &IWDT->CR,     0x00000805 );
}


/**
  * @brief  SVD У
  * @param  void
  * @retval void
  */
void Veri_SVD( void )
{
    CheckSysReg( &SVD->CFGR,    0x00000CB8 );
    CheckSysReg( &SVD->IER,     0x00000000 );
    CheckSysReg( &SVD->VSR,     0x00000004 );
    CheckSysReg( &SVD->CR,      0x00000001 );
}


/* Lin Slave Ӧ */
/**
  * @brief  UART2 У
  * @param  void
  * @retval void
  */
void Veri_UART2( void )
{
    CheckSysReg( &UART2->CSR,  0x00000041 );/* TODO Ҫ֤Ƿάֲ䣨RXʼʹܣӦ󣬽RXʹܽս׶Σ */
    if ((UART2->IER & BIT8) != BIT8 )
    {
        UART2->IER |= ( BIT8 );
    }
    CheckSysReg( &UART2->MCR,  0x00000000 );
    
    if( 0 == CheckNvicIrqEn(UART2_IRQn) )
    {
        NVIC_DisableIRQ(UART2_IRQn);
        NVIC_SetPriority(UART2_IRQn, LIN_UART_IRQ_PRI);             /* жȼ */
        NVIC_EnableIRQ(UART2_IRQn);    
    }
}


/**
  * @brief  LPTIM32 У
  * @param  void
  * @retval void
  */
void Veri_LPTIM32( void )
{
    CheckSysReg( &LPTIM32->CFGR,  0x00000000 );
    CheckSysReg( &LPTIM32->CCSR,  0x00000000 );
    CheckSysReg( &LPTIM32->ARR,  TIMER_LOAD_VALUE - 1 );
    CheckSysReg( &LPTIM32->IER,  0x00000004 );
    CheckSysReg( &LPTIM32->CR,  0x00000001 );
    
    if( 0 == CheckNvicIrqEn(LPTIM_IRQn) )
    {
        NVIC_DisableIRQ(LPTIM_IRQn);
        NVIC_SetPriority(LPTIM_IRQn, 2); 
        NVIC_EnableIRQ(LPTIM_IRQn);
    }     
}


/**
  * @brief  ATIM Ӧ У
  * @param  void
  * @retval void
  */
void Veri_ATIM( void )
{
    CheckSysReg( &ATIM->CR1,  0x00000081 );
    CheckSysReg( &ATIM->CR2,  0x00000000 );
    CheckSysReg( &ATIM->SMCR,  0x00000050 );
    CheckSysReg( &ATIM->DIER,  0x00000018 );
    CheckSysReg( &ATIM->EGR,  0x00000000 );
    CheckSysReg( &ATIM->CCMR1,  0x00000000 );
    CheckSysReg( &ATIM->CCMR2,  0x00000201 );
    CheckSysReg( &ATIM->CCER,  0x00001300 );
    CheckSysReg( &ATIM->ARR,  0xFFFF );
    CheckSysReg( &ATIM->RCR,  0x00000000 );
    CheckSysReg( &ATIM->CCR1,  0x00000000 );
    CheckSysReg( &ATIM->CCR2,  0x00000000 );
    CheckSysReg( &ATIM->BDTR,  0x00000000 );
    CheckSysReg( &ATIM->DCR,  0x00000000 );
    CheckSysReg( &ATIM->BKCR,  0x00000300 );
}


/**
  * @brief  IIC--EEprom У
  * @param  void
  * @retval void
  */
void Veri_I2C( void )
{
    CheckSysReg( &I2C->MSPCFGR,  0x00000001 );
    CheckSysReg( &I2C->MSPIER,  0x00000000 );
    CheckSysReg( &I2C->MSPBGR,  ((I2C_BRG << 16) | I2C_BRG) );
    CheckSysReg( &I2C->MSPTCR,  0x00000032 );
    CheckSysReg( &I2C->SSPCR,  0x00000210 );
    CheckSysReg( &I2C->SSPIER,  0x00000000 );
    
}


/**
  * @brief  SPI2 У
  * @param  void
  * @retval void
  */
void Veri_SPI2( void )
{
    CheckSysReg( &SPI2->CR1,  (0x00000103 | SPI2_BAUD) );
    CheckSysReg( &SPI2->IER,  0x00000000 );
    CheckSysReg( &SPI2->CR2,  0x00000057 );
}


/**
  * @brief  ADC У
  * @param  void
  * @retval void
  */
void Veri_ADC( void )
{
    CheckSysReg( &ADC->IER,  0x00000007 );
    CheckSysReg( &ADC->CFGR,  0x00351181 );
    CheckSysReg( &ADC->SMTR,  0x000000B0 );
    CheckSysReg( &ADC->CHER,  0x000208B8 );
    CheckSysReg( &ADC->CR,  0x00000001 );
    
    if( 0 == CheckNvicIrqEn(ADC_IRQn) )
    {
        NVIC_DisableIRQ(ADC_IRQn);
        NVIC_SetPriority(ADC_IRQn, ADC_IRQ_PRI); 
        NVIC_EnableIRQ(ADC_IRQn);
    }    
}


/**
  * @brief  DMA У
  * @param  void
  * @retval void
  */
void Veri_DMA( void )
{
    CheckSysReg( &DMA->CH4CR,  0x000B281F );
    CheckSysReg( &DMA->GCR,  0x00000001 );
    
    if( 0 == CheckNvicIrqEn(DMA_IRQn) )
    {
        NVIC_DisableIRQ(DMA_IRQn);
        NVIC_SetPriority(DMA_IRQn, DMA_IRQ_PRI); 
        NVIC_EnableIRQ(DMA_IRQn);
    }    
}


/**
  * @brief  BSTIM32 У
  * @param  void
  * @retval void
  */
void Veri_BSTIM32( void )
{
    CheckSysReg( &BSTIM32->CR2,  0x00000020 );
    CheckSysReg( &BSTIM32->IER,  0x00000001 );
    CheckSysReg( &BSTIM32->EGR,  0x00000000 );
    CheckSysReg( &BSTIM32->PSC,  64000 );
    CheckSysReg( &BSTIM32->ARR,  ADC_Captuer_TIME );
    CheckSysReg( &BSTIM32->CR1,  0x00000081 );
    
}


/**
  * @brief  GPTIM0 У
  * @param  void
  * @retval void
  */
void Veri_GPTIM0( void )
{
    CheckSysReg( &GPTIM0->CR1,  0x00000081 );
    CheckSysReg( &GPTIM0->CR2,  0x00000000 );
    CheckSysReg( &GPTIM0->SMCR,  0x00000054 );
    CheckSysReg( &GPTIM0->DIER,  0x00000006 );
    CheckSysReg( &GPTIM0->EGR,  0x00000000 );
    CheckSysReg( &GPTIM0->CCMR1,  0x00000201 );
    CheckSysReg( &GPTIM0->CCMR2,  0x00000000 );
    CheckSysReg( &GPTIM0->CCER,  0x00000031 );
    CheckSysReg( &GPTIM0->PSC,  __SYSTEM_CLOCK_1MHz_PRE - 1 );
    CheckSysReg( &GPTIM0->ARR,  GPTIM0_AUTORELOAD_VALUE - 1 );
    CheckSysReg( &GPTIM0->DCR,  0x00000000 );
    CheckSysReg( &GPTIM0->ITRSEL,  0x00000000 );
}


/**
  * @brief  GPTIM1 У
  * @param  void
  * @retval void
  */
void Veri_GPTIM1( void )
{
    CheckSysReg( &GPTIM1->CR1,  0x00000081 );
    CheckSysReg( &GPTIM1->CR2,  0x00000000 );
    CheckSysReg( &GPTIM1->SMCR,  0x00000000 );
    CheckSysReg( &GPTIM1->DIER,  0x00000000 );
    CheckSysReg( &GPTIM1->EGR,  0x00000000 );
    CheckSysReg( &GPTIM1->CCMR1,  0x00000000 );
    CheckSysReg( &GPTIM1->CCMR2,  0x00006000 );
    CheckSysReg( &GPTIM1->CCER,  0x00001000 );
    CheckSysReg( &GPTIM1->DCR,  0x00000000 );
    CheckSysReg( &GPTIM1->ITRSEL,  0x00000000 );
}


/**
  * @brief  ϵͳĴУ
  * @param  void
  * @retval void
  */
void Veri_Sys_Reg( void )
{
    Veri_Pad();
    
    /* XTHFͣʱʱУ */
    if(SYSCLK_NORMAL == SYSCLKState)
    {
        Veri_CMU();
    }
    
    Veri_FLASH();
    Veri_PMU();
    Veri_RMU();
    Veri_IWDT();
    Veri_SVD();
    Veri_UART2();
    Veri_ATIM();
    Veri_LPTIM32();
    Veri_I2C();
    Veri_SPI2();
    Veri_ADC();
    Veri_DMA();
    Veri_BSTIM32();
    Veri_GPTIM0();
    Veri_GPTIM1();
}


/**
  * @brief  CRCУ-16Bit
  * @param  bufУĿ껺ַ
  *         lenݳ
  * @retval CRC16 У
  */
unsigned short Crc_16Bits( uint8_t *buf, uint8_t len ) 
{
    uint8_t i, j;
    unsigned short current,CRC16;
            
    CRC16 = 0x0000;
    
    for( i=0; i<len; i++ )
    {
        current = buf[i];
        for (j = 0; j < 8; j++) 
        { 
            if ((CRC16 ^ current) & 0x0001)
                CRC16 = (CRC16 >> 1) ^ 0x8408;
            else 
                CRC16 >>= 1; 
            current >>= 1;            
        }
    }
    
    return CRC16;
}


/**
  * @brief  CRCУֽڴ
  * @param  bufĿ껺ַ
  *         lenݳ
  * @retval void
  */
void Crc_16Bits_Add2( uint8_t *buf, uint8_t len ) 
{
    unsigned short Result;
    
    Result  = Crc_16Bits( buf, len );
    buf[len] = Result;                    /* ֽ */
    buf[len+1] = Result>>8;               /* ֽ */
}


/**
  * @brief  CRCУУ
  * @param  bufĿ껺ַ
  *         lenݳ
  * @retval 0crcУȷ
  *         1crcУ
  */
uint8_t Crc_Check( uint8_t *buf, uint8_t len ) 
{
    uint8_t i;

    for( i=0; i<len; i++ )
    {
        if ( buf[i] != 0 ) break;
    }
    if( i == len ) return 1;        /* ݾΪ0,ΪУ */
    
    if( Crc_16Bits( buf, len ) ) return 1;    /* crcУ */
    
    return 0;                /* Уȷ */
}


/**
  * @brief  EEPROM У
  * @param  PageҳƫƵַ
  *         bufĿ껺ַ
  *         lenݳ
  * @retval 0Уȷ
  *         1У
  */
uint8_t Verify_EEprom_para(uint8_t Page ,uint8_t *Buf, uint8_t Len )/* EEpromһжУ麯 */
{
    if( Crc_Check( Buf, Len+2 ) == 0 ) return 0;    /* ramcrcȷ */
    
    if( I2C_Read_Eeprom(I2C, Page, 0x00, Buf, Len ) == 0 ) return 0;/* Ѿeeָ */
    
    if( I2C_Read_Eeprom(I2C, Page, 0x20, Buf, Len ) != 0 ) return 1;/* ȫ */
    
    I2C_Write_Eeprom(I2C, Page, 0x00, Buf, Len );    /* eeݻָ,ͬʱд */
    
    return 0;
}


/**
  * @brief  UserFlash У
  * @param  PageҳƫƵַ
  *         offsetҳƫƵַ
  *         bufĿ껺ַ
  *         lenݳ
  * @retval 0Уȷ
  *         1У
  */
uint8_t Verify_UserFlash_para(uint32_t Page, uint16_t offset, uint8_t *Buf, uint16_t Len)/* DataFlashһжУ麯 */
{
    if( Crc_Check( Buf, Len+2 ) == 0 ) return 0;    /* ramcrcȷ */
    
    if( Read_UserFlash(Page, offset, Buf, Len ) == 0 ) return 0;/* ѾDataFlashָ */
    
    if( Read_UserFlash(Page + USER_FLASH_BKUP_PAGE, offset, Buf, Len ) != 0 ) return 1;/* ȫ */
    
    Write_UserFlash(Page, offset, Buf, Len );    /* DataFlashݻָ,ͬʱд */
    
    return 0;
}

/**
  * @brief  EEPROM У
  * @param  void
  * @retval void
  */
void Ctrldata_EEProm_Verify(void)
{
    if( Verify_EEprom_para( ee_ctrldata_page, (uint8_t *)g_app_struct.ctrl_data.data, ee_ctrldata_lenth ) )
    {
        memset( (void *)g_app_struct.ctrl_data.data, 0x00, 8 );/* Ĭֵ */
    }
}

/**
  * @brief  UserFlash У
  * @param  void
  * @retval void
  */
void Ctrldata_UserFlash_Verify(void)
{
    if( Verify_UserFlash_para( userflash_ctrldata_page, userflash_ctrldata_offset, (uint8_t *)g_app_struct.ctrl_data.data, userflash_ctrldata_lenth ) )
    {
        memset( (void *)g_app_struct.ctrl_data.data, 0x00, 8 );/* Ĭֵ */
    }
}






